@@ -79,6 +79,7 @@ QString REnvironmentObject::makeChildName (const QString &short_child_name, bool
 	if (type & ToplevelEnv) {
 /* Some items are placed outside of their native namespace. E.g. in package:boot item "motor". It can be retrieved using as.environment ("package:boot")$motor. This is extremly ugly. We need to give them (and only them) this special treatment. */
 // TODO: hopefully one day operator "::" will work even in those cases. So check back later, and remove after a sufficient amount of backwards compatibility time
+// NOTE: This appears to have been fixed in R 2.14.0, when all packages were forced to have namespaces.
 		if ((type & PackageEnv) && (!misplaced)) return (packageName () + "::" + safe_name);
 		return (getFullName () + '$' + safe_name);
 	}
@@ -89,9 +90,8 @@ QString REnvironmentObject::makeChildName (const QString &short_child_name, bool
 QString REnvironmentObject::makeChildBaseName (const QString &short_child_name) const {
 	RK_TRACE (OBJECTS);
 
-	if (type & ToplevelEnv) {
-		return (short_child_name);
-	}
+	if (type & ToplevelEnv) return (short_child_name);
+	if (isPackageNamespace ()) return (static_cast<REnvironmentObject*>(parent)->packageName () + ":::" + short_child_name);
 	return (name + '$' + short_child_name);
 }
 
@@ -139,7 +139,10 @@ void REnvironmentObject::updateFromR (RCommandChain *chain, const QStringList &c
 	for (int i = current_symbols.count () - 1; i >= 0; --i) {
 		RObject *child = findChildByName (current_symbols[i]);
 		if (!child) child = createPendingChild (current_symbols[i], i, false, false);
-		if (child->isPending ()) child->updateFromR (chain);
+		if (child->isPending ()) {
+			child->type -= RObject::Pending;	// HACK: Child is not actually pending: We've seen it!
+			child->updateFromR (chain);
+		}
 	}
 
 	RK_ASSERT ((debug_baseline + current_symbols.count ()) == childmap.size ());
@@ -154,15 +157,16 @@ bool REnvironmentObject::updateStructure (RData *new_data) {
 		if (!RObject::updateStructure (new_data)) return false;
 	}
 
+	RData::RDataStorage new_data_data = new_data->structureVector ();
 	if (new_data->getDataLength () > StorageSizeBasicInfo) {
-		RData *children_sub = new_data->getStructureVector ()[StoragePositionChildren];
+		RData *children_sub = new_data_data.at (StoragePositionChildren);
 		RK_ASSERT (children_sub->getDataType () == RData::StructureVector);
 		updateChildren (children_sub);
 
 		// a namespace to go with that?
 		if (new_data->getDataLength () > (StorageSizeBasicInfo + 1)) {
 			RK_ASSERT (new_data->getDataLength () == (StorageSizeBasicInfo + 2));
-			updateNamespace (new_data->getStructureVector ()[StoragePositionNamespace]);
+			updateNamespace (new_data_data.at (StoragePositionNamespace));
 		} else updateNamespace (0);
 	} else {
 		RK_ASSERT (false);
@@ -187,7 +191,7 @@ void REnvironmentObject::updateNamespace (RData* new_data) {
 		added = true;
 		RKGlobals::tracker ()->lockUpdates (true);
 	}
-	namespace_envir->updateStructure (new_data->getStructureVector ()[0]);
+	namespace_envir->updateStructure (new_data->structureVector ().at (0));
 	if (added) {
 		RKGlobals::tracker ()->lockUpdates (false);
 
@@ -200,16 +204,6 @@ void REnvironmentObject::updateNamespace (RData* new_data) {
 	}
 }
 
-void REnvironmentObject::renameChild (RObject *object, const QString &new_name) {
-	RK_TRACE (OBJECTS);
-
-	if (type & GlobalEnv) {
-		RContainerObject::renameChild (object, new_name);
-	} else {
-		RK_ASSERT (false);
-	}
-}
-
 QString REnvironmentObject::renameChildCommand (RObject *object, const QString &new_name) const {
 	RK_TRACE (OBJECTS);
 
