@@ -63,9 +63,18 @@ bool RKModificationTracker::removeObject (RObject *object, RKEditor *editor, boo
 
 	if (!object->isPseudoObject ()) {
 		if (removed_in_workspace) {
-			if (ed) {
+			if (ed && (ed->getObject () == object)) {	// NOTE: do not allow restoring of columns in a data.frame this way. See http://www.mail-archive.com/rkward-devel@lists.sourceforge.net/msg01731.html and replies.
 				if (KMessageBox::questionYesNo (0, i18n ("The object '%1' was removed from workspace or changed to a different type of object, but is currently opened for editing. Do you want to restore it?", object->getFullName ()), i18n ("Restore object?")) == KMessageBox::Yes) {
 					ed->restoreObject (object);
+					/* TODO: It would make a lot of sense to allow restoring to a different name, and possibly different location. This may need some thinking. Probably something like:
+					 * 	object->parentObject ()->removeChildNoDelete (parent);
+					 * 	object->setParentObject (RObjectList::getGlobalEnv ());
+					 * 	// make sure new_name is unique in new parent!
+					 * 	RObjectList::getGlobalEnv ()->insertChild (object, -1);
+					 * 	object->setName (new_name);
+					 * along with proper begin/endAdd/RemoveRows().
+					 * Oh, and listener notifications. That might be tricky.
+					 * */
 					return false;
 				}
 			}
