@@ -2,7 +2,7 @@
                           rkcomponent  -  description
                              -------------------
     begin                : Tue Dec 13 2005
-    copyright            : (C) 2005, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2006, 2007, 2009, 2010, 2011, 2012 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -23,6 +23,7 @@
 #include <qwidget.h>
 
 class RKComponentPropertyBase;
+class RKStandardComponent;
 
 /** a very low level base for RKComponent and RKComponentProperty. */
 class RKComponentBase {
@@ -40,6 +41,7 @@ public:
 		PropertyRObjects = 5,
 		PropertyCode = 6,
 		PropertyLogic = 7,
+		PropertyStringList = 8,
 		PropertyUser = 1000,		/**< for user expansion */
 		PropertyEnd = 1999,
 		ComponentBase = 2001,
@@ -58,6 +60,7 @@ public:
 		ComponentPreviewBox = 2014,
 		ComponentSaveObject = 2015,
 		ComponentFrame = 2016,
+		ComponentOptionSet = 2017,
 		ComponentStandard = 2100,
 		ComponentContextHandler = 2900,
 		ComponentUser = 3000	/**< for user expansion */
@@ -101,7 +104,7 @@ public:
 /** set to required: will only be satisfied if it is valid (and all it's children). Else: always satisfied (but subclasses might override to always be dissatisfied on really bad values. By default RKComponentBase is required at construction */
 	void setRequired (bool require) { required = require; };
 /** simple convenience function to add a child to the map of children */
-	void addChild (const QString &id, RKComponentBase *child);
+	virtual void addChild (const QString &id, RKComponentBase *child);
 
 /** serialize the state of this component / property and all its children. Note: Only the non-internal property-values are serialzed, not the components / properties themselves. @see fetchPropertyValuesRecursive() */
 	QString serializeState () const;
@@ -120,6 +123,7 @@ protected:
 @param include_top_level include direct properties of the component in the list (or only properties of children)
 @param prefix used during recursion to provide full ids for the added objects */
 	void fetchPropertyValuesRecursive (QMap<QString, QString> *list, bool include_top_level=false, const QString &prefix=QString::null) const;
+	friend class RKComponentBuilder;
 /** counterpart to fetchPropertyValuesRecursive (). Tries to apply all values from the list to properties of the given names. If some keys can not be found, or do not resolve to properties, the are ignored.
 @param list a list of id->value such as generated by fetchPropertyValuesRecursive () */
 	void setPropertyValues (QMap<QString, QString> *list, bool warn_internal=false);
@@ -175,11 +179,10 @@ public:
 /** convenience call to set requiredness property (and hence requiredness of this component) */
 	void setRequired (bool required);
 
-/** The parent of this component. Should be notified, whenever isSatisfied () or isReady ()-state changed. */
+/** The parent of this component. Should be notified, whenever isSatisfied () or recursiveStatus () changed. */
 	RKComponent *parentComponent () const { return _parent; };
-
-/** Is the component "ready"? I.e. it is up to date according to current settings. Does not imply it is also satisfied. Default implementation always returns true. */
-	virtual bool isReady () { return true; };
+/** The standard component containing this component (if any). If @param id_adjust is given, it will be set to a relative path to the standard component. */
+	RKStandardComponent *standardComponent (QString *id_adjust=0);
 
 /** Find the id of this component. NOTE: this is slow. Better to store the id in the first place, if needed */
 	QString getIdInParent () const;
@@ -196,8 +199,8 @@ protected:
 	RKComponent *_parent;
 /** usually happens during construction, so you don't need to call this - unless you're RKStandardComponent, and discard the children at some point of time */
 	void createDefaultProperties ();
-/** This function is needed internally, to set the Qt enabledness of this widget, and all child component widgets. Note that the enabledness as stored in the enabledness_property is not necessarily the same as the enabledness in the GUI (and is not affected by this call). In general, a component is enabled in the GUI, if and only if both it's enabledness_property is set to true, and the its parent widget is enabled in Qt. */
-	void updateEnablednessRecursive ();
+/** This function is needed internally, to set the Qt enabledness of this widget, and all child component widgets. Note that the enabledness as stored in the enabledness_property is not necessarily the same as the enabledness in the GUI (and is not affected by this call). In general, a component is enabled in the GUI, if and only if a) it's enabledness_property is set to true, b) its parent widget is enabled in Qt, and c) it's parent component is also enabled. */
+	void updateEnablednessRecursive (bool parent_component_enabled);
 };
 
 #endif
