@@ -2,7 +2,7 @@
                           rkcomponentproperties  -  description
                              -------------------
     begin                : Fri Nov 25 2005
-    copyright            : (C) 2005, 2006, 2007, 2009 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2006, 2007, 2009, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -69,6 +69,35 @@ protected:
 };
 
 
+/////////////////////////////////////////////// StringList /////////////////////////////////////////////////////
+/** RKComponentProperty, which can handle lists of strings, better */
+class RKComponentPropertyStringList : public RKComponentPropertyBase {
+public:
+	RKComponentPropertyStringList (QObject *parent, bool required);
+/** destructor */
+	~RKComponentPropertyStringList ();
+/** for RTTI. see RKComponentBase::RKComponentTypes */
+	int type () { return PropertyStringList; };
+/** change separator string when concatenating strings for value (). Default separator string is "\n" */
+	void setSeparator (const QString &separator) { sep = separator; doChange (); };
+/** reimplemented to return all current strings joined by current separator (setSeparator) */
+	QString value (const QString &modifier=QString::null);
+/** return the string at the given index */
+	const QString valueAt (int index) const { return storage.value (index); };
+/** set the values in string form (values will be split by the current separator)
+@returns false if the value is illegal (in this property, all strings are legal) */
+	bool setValue (const QString &string);
+/** change only the string at the given index. List will be expanded, as necessary. */
+	void setValueAt (int index, const QString &value);
+/** get all current strings as a QStringList */
+	const QStringList& values () const { return storage; };
+/** set current strings as a QStringList */
+	void setValues (const QStringList &new_values) { storage = new_values; doChange (); };
+private:
+	void doChange () { _value.clear (); emit (valueChanged (this)); };
+	QString sep;
+	QStringList storage;
+};
 
 ///////////////////////////////////////////////// Bool ////////////////////////////////////////////////////////
 
@@ -246,9 +275,8 @@ public:
 	bool setObjectValue (RObject *object);
 /** set all the objects in the given new list (if valid), and only those. Emit a signal if there was any change */
 	void setObjectList (const RObject::ObjectList &newlist);
-/** Check whether an object is valid for this property.
-@returns false if the object does not qualify as a valid selection according to current settings (class/type/dimensions), true otherwise */
-	bool isObjectValid (RObject *object);
+/** Return a string describing the problems with the item at position list_index in the objectList(). An empty string, if there is nothing wrong with the object. */
+	QString objectProblems (int list_index) const;
 /** Get current object. If the property can hold several objects, only the first is returned. See objectList ().
 @returns 0 if no valid object is selected */
 	RObject *objectValue ();
@@ -279,11 +307,17 @@ protected:
 /** reimplemented from RObjectListener::objectMetaChanged (). This is so we get notified if the object currently selected is changed */
 	void objectMetaChanged (RObject *changed);
 private:
-/** check all objects currently in the list for validity. Remove invalid objects. Determine validity state depending on how many (valid) objects remain in the list. If the list was changed during validation, and silent!=false a valueChanged () signal is emitted */
+/** check all objects currently in the list for validity. And set validity state accordingly. */
 	void validizeAll (bool silent=false);
-/** simple helper function: Check whether the number of objects currently selected (and only that!), and set the valid state accordingly */
-	void checkListLengthValid ();
+/** simple helper function: Check whether the the list is valid (*after* each object had been validized!), and set the valid state accordingly */
+	void updateValidity ();
+/** internal helper to add the object (and check it for problems).
+ * @returns true, if the list was changed, false, if the object was already in the list or is 0. */
+	bool appendObject (RObject *object);
+/** Check any object for problems */
+	QString checkObjectProblems (RObject *object) const;
 	RObject::ObjectList object_list;
+	QMap<RObject*, QString> problems;
 	int dims;
 	int min_length;
 	int max_length;
