@@ -2,7 +2,7 @@
                           rkstandardcomponent  -  description
                              -------------------
     begin                : Sun Feb 19 2006
-    copyright            : (C) 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2006, 2007, 2009, 2010, 2011, 2012 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -54,6 +54,7 @@
 #include "rktext.h"
 #include "rktabpage.h"
 #include "rkpluginframe.h"
+#include "rkoptionset.h"
 
 #include "../rkglobals.h"
 
@@ -75,11 +76,15 @@ RKStandardComponent::RKStandardComponent (RKComponent *parent_component, QWidget
 	addChild ("code", code = new RKComponentPropertyCode (this, true));		// do not change this name!
 	code->setInternal (true);
 
-	RKComponentPropertyBase *current_object_property = new RKComponentPropertyBase (this, false);
+	RKComponentPropertyRObjects *current_object_property = new RKComponentPropertyRObjects (this, false);
+	RKComponentPropertyRObjects *current_dataframe_property = new RKComponentPropertyRObjects (this, false);
 	current_object_property->setInternal (true);
+	current_dataframe_property->setInternal (true);
 	RKMDIWindow *w = RKWorkplace::mainWorkplace ()->activeWindow (RKMDIWindow::AnyWindowState);
 	if (w) current_object_property->setValue (w->globalContextProperty ("current_object"));
+	if (current_object_property->objectValue () && current_object_property->objectValue ()->isDataFrame ()) current_dataframe_property->setObjectValue (current_object_property->objectValue ());
 	addChild ("current_object", current_object_property);
+	addChild ("current_dataframe", current_dataframe_property);
 
 	// open the main description file for parsing
 	XMLHelper* xml = XMLHelper::getStaticHelper ();
@@ -162,7 +167,7 @@ RKStandardComponent::RKStandardComponent (RKComponent *parent_component, QWidget
 RKStandardComponent::~RKStandardComponent () {
 	RK_TRACE (PLUGIN);
 
-	if (gui) gui->deleteLater ();
+	if (gui) delete gui;	// NOTE: *NOT* using gui->deleteLater (). Destructing the GUI immediately is necessary to get rid of child components, immediately. Otherwise these could try to access their (destroyed) parent, e.g. if they have a timer running that gets triggered before the deletion event arrives.
 	backend->destroy ();	// it will self-destruct, when it has closed the process.
 }
 
@@ -222,15 +227,18 @@ bool RKStandardComponent::createTopLevel (const QDomElement &doc_element, int fo
 
 	dialog_element = xml->getChildElement (doc_element, "dialog", DL_INFO);
 	wizard_element = xml->getChildElement (doc_element, "wizard", DL_INFO);
-	if (!wizard_element.isNull ()) {
-		build_wizard = xml->getBoolAttribute (wizard_element, "recommended", false, DL_INFO);
-	}
 
 	if (force_mode == 0) {
-		if (RKSettingsModulePlugins::getInterfacePreference () == RKSettingsModulePlugins::PreferDialog) {
-			if (!dialog_element.isNull ()) build_wizard = false;
-		} else if (RKSettingsModulePlugins::getInterfacePreference () == RKSettingsModulePlugins::PreferWizard) {
-			if (!wizard_element.isNull ()) build_wizard = true;
+		if (wizard_element.isNull ()) build_wizard = false;
+		else if (dialog_element.isNull ()) build_wizard = true;
+		else {	// both are given
+			if (RKSettingsModulePlugins::getInterfacePreference () == RKSettingsModulePlugins::PreferDialog) {
+				build_wizard = false;
+			} else if (RKSettingsModulePlugins::getInterfacePreference () == RKSettingsModulePlugins::PreferWizard) {
+				build_wizard = true;
+			} else {
+				build_wizard = xml->getBoolAttribute (wizard_element, "recommended", false, DL_INFO);
+			}
 		}
 	} else if (force_mode == 1) {
 		build_wizard = false;
@@ -398,13 +406,6 @@ void RKStandardComponent::handleChange () {
 	RKComponent::changed ();
 }
 
-bool RKStandardComponent::isReady () {
-	RK_TRACE (PLUGIN);
-	RK_ASSERT (backend);
-
-	return (!(backend->isBusy ()));
-}
-
 void RKStandardComponent::backendIdle () {
 	RK_TRACE (PLUGIN);
 
@@ -498,7 +499,7 @@ void RKStandardComponent::addComponentToCurrentPage (RKComponent *component) {
 
 #include <qpushbutton.h>
 
-RKComponentBuilder::RKComponentBuilder (RKStandardComponent *parent_component, const QDomElement &document_element) {
+RKComponentBuilder::RKComponentBuilder (RKComponent *parent_component, const QDomElement &document_element) {
 	RK_TRACE (PLUGIN);
 	parent = parent_component;
 	doc_elem = document_element;
@@ -649,13 +650,23 @@ void RKComponentBuilder::buildElement (const QDomElement &element, QWidget *pare
 			} else {
 				xml->displayError (&e, QString ("Could not embed component '%1'. Not found").arg (component_id), DL_ERROR);
 			}
+		} else if (e.tagName () == "optionset") {
+			widget = new RKOptionSet (e, component (), parent_widget);
+		} else if (e.tagName () == "optiondisplay") {
+			RKComponent *set = component ()->parentComponent ();
+			if (set->type () == RKComponentBase::ComponentOptionSet) {
+				widget = static_cast<RKOptionSet *> (set)->createDisplay (xml->getBoolAttribute (e, "index", true, DL_INFO), parent_widget);
+			} else {
+				xml->displayError (&e, QString ("optiondisplay element is not allowed outside of an optionset"), DL_ERROR);
+			}
 		} else if (e.tagName () == "scriptable") {
 			widget = new RKComponent (component (), parent_widget);
 			QVBoxLayout *layout = new QVBoxLayout (widget);
 			layout->setContentsMargins (0, 0, 0, 0);
 			KVBox *box = new KVBox (widget);
 			layout->addWidget (box);
-			component ()->scriptingProxy ()->addScriptableWidget (id, widget);
+			QString id_adjust;
+			parent->standardComponent (&id_adjust)->scriptingProxy ()->addScriptableWidget (id_adjust + id, widget);
 		} else {
 			xml->displayError (&e, QString ("Invalid tagname '%1'").arg (e.tagName ()), DL_ERROR);
 		}
@@ -725,7 +736,7 @@ void RKComponentBuilder::parseLogic (const QDomElement &element) {
 	if (!e.isNull ()) {
 		QString file = xml->getStringAttribute (e, "file", QString (), DL_INFO);
 		QString inline_command = e.text ();
-		component ()->scriptingProxy ()->initialize (file, inline_command);
+		parent->standardComponent ()->scriptingProxy ()->initialize (file, inline_command);
 	}
 }
 
