@@ -2,7 +2,7 @@
                           rkstandardcomponentgui  -  description
                              -------------------
     begin                : Sun Mar 19 2006
-    copyright            : (C) 2006, 2007, 2009 by Thomas Friedrichsmeier
+    copyright            : (C) 2006, 2007, 2009, 2012 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -256,15 +256,18 @@ void RKStandardComponentGUI::updateCode () {
 
 void RKStandardComponentGUI::updateCodeNow () {
 	RK_TRACE (PLUGIN);
+	static bool was_valid = false;
+	if (was_valid) code_display->saveScrollPosition ();
 
-	code_display->setReadOnly (false);	// sigh. The kate part does not allow setting text programatically, when it is read only
 	if (!code_property->isValid ()) {
 		code_display->setText (i18n ("Processing. Please wait"));
 		RK_DO (qDebug ("code not ready to be displayed: pre %d, cal %d, pri %d", !code_property->preprocess ().isNull (), !code_property->calculate ().isNull (), !code_property->printout ().isNull ()), PLUGIN, DL_DEBUG);
+		was_valid = false;
 	} else {
 		code_display->setText ("local({\n" + code_property->preprocess () + code_property->calculate () + code_property->printout () + "})\n");
+		code_display->restoreScrollPosition ();
+		was_valid = true;
 	}
-	code_display->setReadOnly (true);
 }
 
 ///////////////////////////////// RKStandardComponentWizard /////////////////////////////////
@@ -306,7 +309,7 @@ void RKStandardComponentWizard::createWizard (bool switchable) {
 	cancel_button = new QPushButton (i18n ("Cancel"), this);
 	main_grid->addWidget (cancel_button, 2, 0, Qt::AlignLeft);
 	if (enslaved) cancel_button->hide ();
-	help_button = new QPushButton ("Help", this);
+	help_button = new QPushButton (i18n ("Help"), this);
 	main_grid->addWidget (help_button, 2, 1, Qt::AlignLeft);
 	prev_button = new QPushButton (QString::null, this);
 	prev_button->setEnabled (false);
@@ -498,8 +501,9 @@ RKComponent *RKStandardComponentStack::addPage (RKComponent *parent) {
 	RK_TRACE (PLUGIN);
 
 	PageDef *def = new PageDef;
-	def->page = new RKComponent (parent, this);
-	setCurrentIndex (addWidget (def->page));
+	KVBox *wrap = new KVBox (this);
+	def->page = new RKComponent (parent, wrap);
+	setCurrentIndex (addWidget (wrap));
 	pages.append (def);
 
 	return def->page;
