@@ -2,7 +2,7 @@
                           rkvarslot.cpp  -  description
                              -------------------
     begin                : Thu Nov 7 2002
-    copyright            : (C) 2002, 2007, 2008, 2009 by Thomas Friedrichsmeier
+    copyright            : (C) 2002, 2007, 2008, 2009, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -74,10 +74,11 @@ RKVarSlot::RKVarSlot (const QDomElement &element, RKComponent *parent_component,
 
 		// make it look like a line-edit
 		list->header ()->hide ();
-		QTreeWidgetItem* dummy = new QTreeWidgetItem (list);
-		dummy->setText (0, " ");
-		list->setFixedHeight (list->visualItemRect (dummy).height () + 2*list->visualItemRect (dummy).top () + 4);
-		delete dummy;
+		QTreeWidgetItem dummy (list);
+		dummy.setText (0, "Tg");
+		int top, left, bottom, right;
+		list->getContentsMargins (&left, &top, &right, &bottom);
+		list->setFixedHeight (list->visualItemRect (&dummy).height () + 2*list->visualItemRect (&dummy).top () + top + bottom);
 		list->header ()->setStretchLastSection (true);
 		list->hideColumn (0);
 		list->setHorizontalScrollBarPolicy (Qt::ScrollBarAlwaysOff);
@@ -130,14 +131,18 @@ void RKVarSlot::availablePropertyChanged (RKComponentPropertyBase *) {
 	RK_DO (qDebug ("contained in varslot: %s", available->value ().toLatin1 ().data ()), PLUGIN, DL_DEBUG);
 
 	RObject::ObjectList objlist = available->objectList ();
-	RObject::ObjectList::const_iterator it = objlist.begin ();
-	int i = 1;
-	while (it != objlist.end ()) {
+	for (int i = 0; i < objlist.count (); ++i) {
+		RObject *object = objlist[i];
+
 		QTreeWidgetItem *new_item = new QTreeWidgetItem (list);
-		new_item->setText (0, QString::number (i++));
-		new_item->setText (1, (*it)->getShortName ());
-		item_map.insert (new_item, *it);
-		++it;
+		new_item->setText (0, QString::number (i + 1));
+		new_item->setText (1, object->getShortName ());
+		QString probs = available->objectProblems (i);
+		if (!probs.isEmpty ()) {
+			new_item->setToolTip (1, i18n ("<p>This object is not allowed, here, for the following reason(s):</p>") + probs);
+			new_item->setIcon (1, RKStandardIcons::getIcon (RKStandardIcons::ActionDeleteVar));
+		}
+		item_map.insert (new_item, object);
 	}
 	if (multi) list->resizeColumnToContents (0);
 
