@@ -59,13 +59,13 @@ function calculate () {
 		echo ('}\n');
 	}
 	echo ('\n');
-	echo (object + ' <<- data		# assign to globalenv()\n');
+	echo ('.GlobalEnv$' + object + ' <- data		# assign to globalenv()\n');
 	if (getValue ("doedit") && data_frame) {
-		echo ('rk.edit (' + object + ')\n');
+		echo ('rk.edit (.GlobalEnv$' + object + ')\n');
 	}
 }
 
 function printout () {
-	makeHeaderCode ("Import SPSS data", new Array("File",  getValue ("file"), "Import as",  getValue ("saveto")));
+	makeHeaderCode ("Import SPSS data", new Array("File",  getValue ("file"), "Import as", getValue ("saveto")));
 }
 
