@@ -11,6 +11,11 @@ function prepareLabel (labelname) {
 	return label;
 }
 
+function preprocess () {
+	if (getValue ("custom_margins.checked")) {
+		echo ("par (mar=c (" + getValue ("margin_bottom") + ", " + getValue ("margin_left") + ", " + getValue ("margin_top") + ", " + getValue ("margin_right") + "))\n");
+	}
+}
 
 function calculate () {
 	if (getValue ("grid_enable") == "true") {
@@ -39,6 +44,8 @@ function printout () {
 	var type = "";
 	var col = "";
 	var asp = "";
+	var las = "";
+	var cexaxis = "";
 	var options = "";
 
 	xvars = getValue ("xvar").split ("\n");
@@ -102,6 +109,15 @@ function printout () {
 	}
 
 
+	// las : axis tick label orientation
+	las = getValue ("ticklblrot")
+	if (las == "") las = getValue ("default_ticklblrot");
+	if (las != "") las = ", las=" + las;
+
+	// cex.axis : axis tick label scale
+	cexaxis = getValue ("cexaxis")
+	if (cexaxis != "") cexaxis = ", cex.axis=" + cexaxis;
+
 	// final touches
 	if (log != "") log = ", log=\"" + log + "\"";
 
@@ -122,7 +138,7 @@ function printout () {
 	else asp = "";
 
 	// make option string
-	options = type + col + xaxt + yaxt + log + xlim + ylim + xlab + ylab + main + sub + asp;
+	options = type + col + xaxt + yaxt + log + xlim + ylim + xlab + ylab + main + sub + asp + las + cexaxis;
 
 	echo (options);
 }
