@@ -36,6 +36,7 @@
 #include "../agents/showedittextfileagent.h"
 #include "../agents/rkeditobjectagent.h"
 #include "../agents/rkprintagent.h"
+#include "../agents/rkdebughandler.h"
 #include "../windows/rcontrolwindow.h"
 #include "../windows/rkworkplace.h"
 #include "../windows/rkcommandlog.h"
@@ -106,6 +107,7 @@ RInterface::RInterface () {
 	issueCommand (fake);
 
 	new RKSessionVars (this);
+	new RKDebugHandler (this);
 	new RKRBackendProtocolFrontend (this);
 	RKRBackendProtocolFrontend::instance ()->setupBackend ();
 
@@ -261,9 +263,7 @@ void RInterface::rCommandDone (RCommand *command) {
 
 	if (command->getFlags () == GET_LIB_PATHS) {
 		RK_ASSERT (command->getDataType () == RData::StringVector);
-		for (unsigned int i = 0; i < command->getDataLength (); ++i) {
-			RKSettingsModuleRPackages::defaultliblocs.append (command->getStringVector ()[i]);
-		}
+		RKSettingsModuleRPackages::defaultliblocs += command->stringVector ();
 
 		RCommandStack *stack = RCommandStack::currentStack ();
 		RCommandChain *chain = stack->currentChain ();
@@ -282,7 +282,7 @@ void RInterface::rCommandDone (RCommand *command) {
 	} else if (command->getFlags () == GET_HELP_BASE) {
 		RK_ASSERT (command->getDataType () == RData::StringVector);
 		RK_ASSERT (command->getDataLength () == 1);
-		RKSettingsModuleR::help_base_url = command->getStringVector ()[0];
+		RKSettingsModuleR::help_base_url = command->stringVector ().value (0);
 	} else if (command->getFlags () == SET_RUNTIME_OPTS) {
 		// no special handling. In case of failures, staturt_fail was set to true, above.
 	} else if (command->getFlags () == STARTUP_PHASE2_COMPLETE) {
@@ -473,7 +473,11 @@ void RInterface::cancelCommand (RCommand *command) {
 	if (!(command->type () & RCommand::Sync)) {
 		command->status |= RCommand::Canceled;
 		if (command->type () && RCommand::Running) {
-			RKRBackendProtocolFrontend::instance ()->interruptCommand (command->id ());
+			if ((RKDebugHandler::instance ()->state () == RKDebugHandler::InDebugPrompt) && (command == RKDebugHandler::instance ()->command ())) {
+				RKDebugHandler::instance ()->sendCancel ();
+			} else {
+				RKRBackendProtocolFrontend::instance ()->interruptCommand (command->id ());
+			}
 		}
 		RCommandStackModel::getModel ()->itemChange (command);
 	} else {
@@ -511,7 +515,7 @@ QStringList RInterface::processPlainGenericRequest (const QStringList &calllist)
 		// if we're still alive, quitting was cancelled
 		return (QStringList ("FALSE"));
 	} else if (call == "preLocaleChange") {
-		int res = KMessageBox::warningContinueCancel (0, i18n ("A command in the R backend is trying to change the character encoding. While RKWard offers support for this, and will try to adjust to the new locale, this operation may cause subtle bugs, if data windows are currently open. Also the feature is not well tested, yet, and it may be advisable to save your workspace before proceeding.\nIf you have any data editor opened, or in any doubt, it is recommended to close those first (this will probably be auto-detected in later versions of RKWard). In this case, please chose 'Cancel' now, then close the data windows, save, and retry."), i18n ("Locale change"));
+		int res = KMessageBox::warningContinueCancel (0, i18n ("A command in the R backend is trying to change the character encoding. While RKWard offers support for this, and will try to adjust to the new locale, this operation may cause subtle bugs, if data windows are currently open. Also the feature is not well tested, yet, and it may be advisable to save your workspace before proceeding.\nIf you have any data editor opened, or in any doubt, it is recommended to close those first (this will probably be auto-detected in later versions of RKWard). In this case, please choose 'Cancel' now, then close the data windows, save, and retry."), i18n ("Locale change"));
 		if (res != KMessageBox::Continue) return (QStringList ("FALSE"));
 	} else if (call == "listPlugins") {
 		RK_ASSERT (calllist.count () == 1);
@@ -542,7 +546,7 @@ QStringList RInterface::processPlainGenericRequest (const QStringList &calllist)
 			RKConsole::mainConsole ()->setCommandHistory (calllist.mid (2), calllist.value (1) == "append");
 		}
 	} else if (call == "getWorkspaceUrl") {
-		KUrl url = RObjectList::getObjectList ()->getWorkspaceURL ();
+		KUrl url = RKWorkplace::mainWorkplace ()->workspaceURL ();
 		if (!url.isEmpty ()) return (QStringList (url.url ()));
 	} else if (call == "workplace.layout") {
 		if (calllist.value (1) == "set") {
@@ -606,6 +610,8 @@ QStringList RInterface::processPlainGenericRequest (const QStringList &calllist)
 		}
 	} else if (call == "printPreview") {
 		RKPrintAgent::printPostscript (calllist.value (1), true);
+	} else if (call == "endBrowserContext") {
+		RKDebugHandler::instance ()->endDebug ();
 	} else {
 		return (QStringList ("Error: unrecognized request '" + call + "'."));
 	}
@@ -766,11 +772,15 @@ void RInterface::processRBackendRequest (RBackendRequest *request) {
 			dummy_command = true;
 		}
 
+
 		bool ok = RKReadLineDialog::readLine (0, i18n ("R backend requests information"), request->params["prompt"].toString (), command, &result);
 		request->params["result"] = QVariant (result);
 
 		if (dummy_command) delete command;
 		if (!ok) request->params["cancelled"] = QVariant (true);
+	} else if (type == RBackendRequest::Debugger) {
+		RKDebugHandler::instance ()->debugCall (request, runningCommand ());
+		return;		// request will be closed by the debug handler
 	} else if ((type == RBackendRequest::ShowFiles) || (type == RBackendRequest::EditFiles)) {
 		ShowEditTextFileAgent::showEditFiles (request);
 		return;		// we are not done, yet!
