@@ -31,7 +31,7 @@
 
 // statics
 RKRBackend *RKRBackend::this_pointer = 0;
-RKRBackend::RKReplStatus RKRBackend::repl_status = { QByteArray (), 0, true, 0, 0, RKRBackend::RKReplStatus::NoUserCommand, 0, false, false };
+RKRBackend::RKReplStatus RKRBackend::repl_status = { QByteArray (), 0, true, 0, 0, RKRBackend::RKReplStatus::NoUserCommand, 0, RKRBackend::RKReplStatus::NotInBrowserContext, false };
 void* RKRBackend::default_global_context = 0;
 
 #include <qstring.h>
@@ -39,7 +39,6 @@ void* RKRBackend::default_global_context = 0;
 #include <QThread>
 #include <QDir>
 #include <qtextcodec.h>
-#include <klocale.h>
 
 #include "../core/robject.h"
 #include "../version.h"
@@ -100,6 +99,23 @@ extern "C" {
 #	include <fcntl.h>
 #endif
 
+///// i18n
+#if (defined ENABLE_NLS) && (!defined RKWARD_THREADED)		// ENABLE_NLS is from Rconfig.h
+#	include <libintl.h>
+#	define RK_MSG_DOMAIN "rkward"
+	// define i18n to be compatible with the easiest usage in KDE
+#	define i18n(msgid) QString::fromUtf8(dgettext(RK_MSG_DOMAIN, msgid))
+	void RK_setupGettext (const char* locale_dir) {
+		bindtextdomain (RK_MSG_DOMAIN, locale_dir);
+		bind_textdomain_codeset (RK_MSG_DOMAIN, "UTF-8");
+	}
+#else
+#	define i18n(msgid) QString(msgid)
+	void RK_setupGettext (const char*) {};
+#endif
+
+
+///// interrupting R
 void RK_scheduleIntr () {
 	RK_DO (qDebug ("interrupt scheduled"), RBACKEND, DL_DEBUG);
 	RKRBackend::repl_status.interrupted = true;
@@ -179,7 +195,7 @@ Rboolean RKToplevelStatementFinishedCallback (SEXP expr, SEXP value, Rboolean su
 	Q_UNUSED (value);
 	Q_UNUSED (visible);
 
-	if ((RKRBackend::repl_status.eval_depth == 0) && (!RKRBackend::repl_status.in_browser_context)) {		// Yes, toplevel-handlers _do_ get called in a browser context!
+	if ((RKRBackend::repl_status.eval_depth == 0) && (!RKRBackend::repl_status.browser_context)) {		// Yes, toplevel-handlers _do_ get called in a browser context!
 		RK_ASSERT (RKRBackend::repl_status.user_command_status = RKRBackend::RKReplStatus::UserCommandRunning);
 		if (succeeded) {
 			RKRBackend::repl_status.user_command_successful_up_to = RKRBackend::repl_status.user_command_parsed_up_to;
@@ -214,8 +230,9 @@ void RKTransmitNextUserCommandChunk (unsigned char* buf, int buflen) {
 
 	bool reached_eof = false;
 	int pos = 0;
+	const int max_pos = buflen - 2;	// one for the termination
 	bool reached_newline = false;
-	while (pos < (buflen-1)) {
+	while (true) {
 		buf[pos] = *current_buffer;
 		if (*current_buffer == '\n') {
 			reached_newline = true;
@@ -225,6 +242,7 @@ void RKTransmitNextUserCommandChunk (unsigned char* buf, int buflen) {
 			reached_eof = true;
 			break;
 		}
+		if (pos >= max_pos) break;
 		++current_buffer;
 		++pos;
 	}
@@ -249,14 +267,14 @@ int RReadConsole (const char* prompt, unsigned char* buf, int buflen, int hist)
 	RK_ASSERT (buf && buflen);
 	RK_ASSERT (RKRBackend::repl_status.eval_depth >= 0);
 
-	if (RKRBackend::repl_status.in_browser_context) {		// previously we were in a browser context. Check, whether we've left that.
+	if (RKRBackend::repl_status.browser_context) {		// previously we were in a browser context. Check, whether we've left that.
 		if (RKRBackend::default_global_context == R_GlobalContext) {
-			RKRBackend::repl_status.in_browser_context = false;
-			RK_ASSERT (!hist);
+			RKRBackend::repl_status.browser_context = RKRBackend::RKReplStatus::NotInBrowserContext;
+			RKRBackend::this_pointer->handlePlainGenericRequest (QStringList ("endBrowserContext"), false);
 		}
 	}
 	
-	if ((!RKRBackend::repl_status.in_browser_context) && (RKRBackend::repl_status.eval_depth == 0)) {
+	if ((!RKRBackend::repl_status.browser_context) && (RKRBackend::repl_status.eval_depth == 0)) {
 		while (1) {
 			if (RKRBackend::repl_status.user_command_status == RKRBackend::RKReplStatus::NoUserCommand) {
 				RCommandProxy *command = RKRBackend::this_pointer->fetchNextCommand ();
@@ -327,7 +345,7 @@ int RReadConsole (const char* prompt, unsigned char* buf, int buflen, int hist)
 				int n_frames = 0;
 				RCommandProxy *dummy = RKRBackend::this_pointer->runDirectCommand ("sys.nframe()", RCommand::GetIntVector);
 				if ((dummy->getDataType () == RData::IntVector) && (dummy->getDataLength () == 1)) {
-					n_frames = dummy->getIntVector ()[0];
+					n_frames = dummy->intVector ().at (0);
 				}
 				// What the ??? Why does this simple version always return 0?
 				//int n_frames = RKRSupport::SEXPToInt (RKRSupport::callSimpleFun0 (Rf_install ("sys.nframe"), R_GlobalEnv));
@@ -355,14 +373,40 @@ int RReadConsole (const char* prompt, unsigned char* buf, int buflen, int hist)
 
 	// here, we handle readline() calls and such, i.e. not the regular prompt for code
 	// browser() also takes us here.
+	QVariantMap params;
+	RBackendRequest::RCallbackType request_type = RBackendRequest::ReadLine;
+	params["prompt"] = QVariant (prompt);
+	params["cancelled"] = QVariant (false);
+
+	// add info for browser requests
 	if (hist && (RKRBackend::default_global_context != R_GlobalContext)) {
-		// TODO: give browser() special handling!
-		RKRBackend::repl_status.in_browser_context = true;
+		if (RKRBackend::repl_status.browser_context == RKRBackend::RKReplStatus::InBrowserContextPreventRecursion) {
+			qstrncpy ((char *) buf, "n\n", buflen);	// skip this, by feeding the browser() a continue
+			return 1;
+		} else {
+			RKRBackend::repl_status.browser_context = RKRBackend::RKReplStatus::InBrowserContextPreventRecursion;
+			RCommandProxy *dummy = RKRBackend::this_pointer->runDirectCommand (".rk.callstack.info()", RCommand::GetStructuredData);
+
+			request_type = RBackendRequest::Debugger;
+			if ((dummy->getDataType () == RData::StructureVector) && (dummy->getDataLength () >= 4)) {
+				RData::RDataStorage dummy_data = dummy->structureVector ();
+				params["calls"] = QVariant (dummy_data.at (0)->stringVector ());
+				params["funs"] = QVariant (dummy_data.at (1)->stringVector ());
+				params["envs"] = QVariant (dummy_data.at (2)->stringVector ());
+				params["locals"] = QVariant (dummy_data.at (3)->stringVector ());
+				params["relsrclines"] = QVariant (dummy_data.at (4)->stringVector ());		// hacky: passing a QList<int> is not supported by QVariant
+			} else {
+				RK_ASSERT (false);
+			}
+
+			RKRBackend::repl_status.browser_context = RKRBackend::RKReplStatus::InBrowserContext;
+		}
+
+		RK_ASSERT (RKRBackend::repl_status.browser_context == RKRBackend::RKReplStatus::InBrowserContext);
 	}
 
-	RBackendRequest request (true, RBackendRequest::ReadLine);
-	request.params["prompt"] = QVariant (prompt);
-	request.params["cancelled"] = QVariant (false);
+	RBackendRequest request (true, request_type);
+	request.params = params;
 
 	RKRBackend::this_pointer->handleRequest (&request);
 	if (request.params["cancelled"].toBool ()) {
@@ -411,7 +455,7 @@ void RWriteConsoleEx (const char *buf, int buflen, int type) {
 	RK_TRACE (RBACKEND);
 
 	// output while nothing else is running (including handlers?) -> This may be a syntax error.
-	if ((RKRBackend::repl_status.eval_depth == 0) && (!RKRBackend::repl_status.in_browser_context) && (!RKRBackend::this_pointer->isKilled ())) {
+	if ((RKRBackend::repl_status.eval_depth == 0) && (!RKRBackend::repl_status.browser_context) && (!RKRBackend::this_pointer->isKilled ())) {
 		if (RKRBackend::repl_status.user_command_status == RKRBackend::RKReplStatus::UserCommandTransmitted) {
 			// status UserCommandTransmitted might have been set from RKToplevelStatementFinishedHandler, too, in which case all is fine
 			// (we're probably inside another task handler at this point, then)
@@ -427,6 +471,7 @@ void RWriteConsoleEx (const char *buf, int buflen, int type) {
 	}
 
 	if (RKRBackend::this_pointer->killed == RKRBackend::AlreadyDead) return;	// this check is mostly for fork()ed clients
+	if (RKRBackend::repl_status.browser_context == RKRBackend::RKReplStatus::InBrowserContextPreventRecursion) return;
 	RKRBackend::this_pointer->fetchStdoutStderr (true);
 	RKRBackend::this_pointer->handleOutput (RKRBackend::this_pointer->current_locale_codec->toUnicode (buf, buflen), buflen, type == 0 ? ROutput::Output : ROutput::Warning);
 }
@@ -827,7 +872,7 @@ LibExtern int R_interrupts_pending;
 SEXP doError (SEXP call) {
 	RK_TRACE (RBACKEND);
 
-	if ((RKRBackend::repl_status.eval_depth == 0) && (!RKRBackend::repl_status.in_browser_context) && (!RKRBackend::this_pointer->isKilled ()) && (RKRBackend::repl_status.user_command_status != RKRBackend::RKReplStatus::ReplIterationKilled)) {
+	if ((RKRBackend::repl_status.eval_depth == 0) && (!RKRBackend::repl_status.browser_context) && (!RKRBackend::this_pointer->isKilled ()) && (RKRBackend::repl_status.user_command_status != RKRBackend::RKReplStatus::ReplIterationKilled)) {
 		RKRBackend::repl_status.user_command_status = RKRBackend::RKReplStatus::UserCommandFailed;
 	}
 	if (RKRBackend::repl_status.interrupted) {
@@ -966,8 +1011,8 @@ bool RKRBackend::startR () {
 	// re-direct stdout / stderr to a pipe, so we can read output from system() calls
 	int pfd[2];
 	pipe (pfd);
-	dup2 (STDOUT_FILENO, STDERR_FILENO);		// single channel to avoid interleaving hell, for now.
 	dup2 (pfd[1], STDOUT_FILENO);
+	dup2 (pfd[1], STDERR_FILENO);		// forward both to a single channel to avoid interleaving hell, for now.
 	close (pfd[1]);
 	stdout_stderr_fd = pfd[0];
 #endif
@@ -1028,7 +1073,7 @@ bool RKRBackend::startR () {
 	// get info on R runtime version
 	RCommandProxy *dummy = runDirectCommand ("as.numeric (R.version$major) * 1000 + as.numeric (R.version$minor) * 10", RCommand::GetIntVector);
 	if ((dummy->getDataType () == RData::IntVector) && (dummy->getDataLength () == 1)) {
-		r_version = dummy->getIntVector ()[0];
+		r_version = dummy->intVector ().at (0);
 	} else {
 		RK_ASSERT (false);
 		r_version = 0;
@@ -1056,16 +1101,18 @@ void completeForkMaster () {
 	if (backend_was_forked) return;
 	backend_was_forked = true;
 
-	// Block SIGCLD in the main thread from now on. I don't fully understand, why, but otherwise, these signals
+	// Block SIGCHLD in the main thread from now on. I don't fully understand, why, but otherwise, these signals
 	// interrupt the select() call in the fork()ing code of library(parallel)
 	sigset_t new_set;
 	sigemptyset (&new_set);
-	sigaddset (&new_set, SIGCLD);
+	sigaddset (&new_set, SIGCHLD);
 	pthread_sigmask (SIG_BLOCK, &new_set, NULL);
 
 //	This was used to show a warning message. Unfortunately, however, forks also occur on every popen (i.e. in system(..., intern=TRUE).
 //	RKRBackend::this_pointer->handlePlainGenericRequest (QStringList ("forkNotification"), false);
 	RK_DO (qDebug ("Backend process forked (for the first time, this session)"), RBACKEND, DL_WARNING);
+//	NOTE: perhaps we can heuristically differentiate from popen by checking sys.calls() for something with "fork" in it. 
+//	esp., in case we discover adverse side-effects of blocking SIGCHLD, we should attempt this
 }
 
 void completeForkChild () {
@@ -1291,12 +1338,12 @@ void RKRBackend::printAndClearCapturedMessages (bool with_header) {
 	catToOutputFile (out);
 }
 
-void RKRBackend::run () {
+void RKRBackend::run (const QString &locale_dir) {
 	RK_TRACE (RBACKEND);
 	killed = NotKilled;
 	previous_command = 0;
 
-	initialize ();
+	initialize (QFile::encodeName (locale_dir));
 
 	enterEventLoop ();
 }
@@ -1432,7 +1479,7 @@ QStringList RKRBackend::handlePlainGenericRequest (const QStringList &parameters
 	return request.params.value ("return").toStringList ();
 }
 
-void RKRBackend::initialize () {
+void RKRBackend::initialize (const char *locale_dir) {
 	RK_TRACE (RBACKEND);
 
 	// in RInterface::RInterface() we have created a fake RCommand to capture all the output/errors during startup. Fetch it
@@ -1444,6 +1491,7 @@ void RKRBackend::initialize () {
 	bool lib_load_fail = false;
 	bool sink_fail = false;
 	if (!runDirectCommand ("library (\"rkward\")\n")) lib_load_fail = true;
+	RK_setupGettext (locale_dir);	// must happen *after* package loading, since R will re-set it
 	if (!runDirectCommand (QString ("stopifnot(.rk.app.version==\"%1\")\n").arg (RKWARD_VERSION))) lib_load_fail = true;
 	if (!runDirectCommand (".rk.fix.assignments ()\n")) sink_fail = true;
 
@@ -1484,14 +1532,14 @@ void RKRBackend::checkObjectUpdatesNeeded (bool check_list) {
 	// TODO: avoid parsing this over and over again
 		RK_DO (qDebug ("checkObjectUpdatesNeeded: getting search list"), RBACKEND, DL_TRACE);
 		RCommandProxy *dummy = runDirectCommand ("search ()\n", RCommand::GetStringVector);
-		if (dummy->getStringVector () != toplevel_env_names) search_update_needed = true;
-		if (search_update_needed) toplevel_env_names = dummy->getStringVector ();
+		if (dummy->stringVector () != toplevel_env_names) search_update_needed = true;
+		if (search_update_needed) toplevel_env_names = dummy->stringVector ();
 		delete dummy;
 	
 	// TODO: avoid parsing this over and over again
 		RK_DO (qDebug ("checkObjectUpdatesNeeded: getting globalenv symbols"), RBACKEND, DL_TRACE);
 		dummy = runDirectCommand ("ls (globalenv (), all.names=TRUE)\n", RCommand::GetStringVector);
-		QStringList new_globalenv_toplevel_names = dummy->getStringVector ();
+		QStringList new_globalenv_toplevel_names = dummy->stringVector ();
 		if (new_globalenv_toplevel_names.count () != global_env_toplevel_names.count ()) {
 			globalenv_update_needed = true;
 		} else {
