@@ -1,12 +1,32 @@
+# check the context in which this package is loaded
+#' @export
+.onAttach <- function(...) {
+	.rk.inside.rkward.session(warn = TRUE)
+}
+
+# this function shall test if the rkward package was loaded in a running RKWard session
+#' @export
+.rk.inside.rkward.session <- function(warn = FALSE){
+	inside.rkward <- is.loaded("rk.do.generic.request")
+	if(isTRUE(warn) & !isTRUE(inside.rkward)){
+		warning("You've loaded the package 'rkward', but RKWard doesn't appear to be running. If this causes trouble, try detach(\"package:rkward\").",
+		call. = FALSE)
+	}
+	return(inside.rkward)
+}
+
+#' @export
 ".rk.get.meta" <- function (x) {
 	y <- attr (x, ".rk.meta");
 	c (names (y), as.character (y))
 }
 
+#' @export
 ".rk.set.meta" <- function (x, m) {
 	eval (substitute (attr (x, ".rk.meta") <<- m))
 }
 
+#' @export
 ".rk.set.invalid.field" <- function (x, row, value) {
 	l <- attr (x, ".rk.invalid.fields");
 	if (is.null (l)) l <- list ();
@@ -15,6 +35,7 @@
 	eval (substitute (attr (x, ".rk.invalid.fields") <<- l))
 }
 
+#' @export
 ".rk.data.frame.insert.row" <- function (x, index=0) {
 	if ((index == 0) || (index > dim (x)[1])) {	# insert row at end
 		eval (substitute (x[dim(x)[1]+1,] <<- c (NA)))
@@ -28,6 +49,7 @@
 }
 
 # TODO: is there a nice way to get rid of a row without removing the meta-data?
+#' @export
 ".rk.data.frame.delete.row" <- function (x, index) {
 	attriblist <- list ()
 	for (i in 1:dim (x)[2]) {
@@ -42,6 +64,7 @@
 }
 
 # function below is only needed to ensure a nice ordering of the columns. Simply adding a new column would be much easier than this.
+#' @export
 ".rk.data.frame.insert.column" <- function (x, label, index=0) {
 	column <- rep (as.numeric (NA), times=dim (x)[1])
 	if ((index == 0) || (index > dim (x)[2])) {	# insert column at end
@@ -56,6 +79,7 @@
 	}
 }
 
+#' @export
 ".rk.do.error" <- function () {
 # comment in R sources says, it may not be good to query options during error handling. But what can we do, if R_ShowErrorMessages is not longer exported?
 	if (getOption ("show.error.messages")) {
@@ -63,18 +87,22 @@
 	}
 }
 
+#' @export
 ".rk.set.reply" <- function (x) .rk.variables$.rk.rkreply <- x
 
+#' @export
 ".rk.do.call" <- function (x, args=NULL) {
 	.rk.set.reply (NULL)
 	.Call ("rk.do.command", c (x, args));
 	return (.rk.variables$.rk.rkreply)
 }
 
+#' @export
 ".rk.do.plain.call" <- function (x, args=NULL, synchronous=TRUE) {
 	.Call ("rk.do.generic.request", c (x, args), isTRUE (synchronous))
 }
 
+#' @export
 ".rk.find.package.pluginmaps" <- function (package, all.maps=FALSE) {
 	if(isTRUE(all.maps)){
 		# look for all pluginmaps in the rkward folder
@@ -92,6 +120,7 @@
 
 # Gather status information on installed and available packages.
 # Return value is used in class RKRPackageInstallationStatus of the frontend
+#' @export
 ".rk.get.package.intallation.state" <- function () {
 	# fetch all status information
 	available <- .rk.cached.available.packages ()
@@ -118,6 +147,7 @@
 
 # package information formats may - according to the help - be subject to change. Hence this function to cope with "missing" values
 # also it concatenates everything to a single vector, so we can easily get the whole structure with a single call
+#' @export
 ".rk.get.installed.packages" <- function () {
 	x <- as.data.frame(installed.packages(fields="Title"))
 	# does a package enhance RKWard, i.e. provide plugins?
@@ -134,6 +164,7 @@
 }
 
 # This function works like available.packages (with no arguments), but does simple caching of the result, and of course uses a cache if available. Cache is only used, if it is less than 1 hour old, and options("repos") is unchanged.
+#' @export
 ".rk.cached.available.packages" <- function () {
 	x <- NULL
 	if (exists ("available.packages.cache", envir=.rk.variables) && (!is.null (.rk.variables$available.packages.cache))) {
@@ -152,49 +183,16 @@
 	return (x)
 }
 
-"require" <- function (package, quietly = FALSE, character.only = FALSE, ...)
-{
-	if (!character.only) {
-		package <- as.character(substitute(package))
-	}
-	if (!base::require(as.character(package), quietly = quietly, character.only = TRUE, ...)) {
-		.rk.do.call("require", as.character(package))
-		invisible(base::require(as.character(package), quietly = TRUE, character.only = TRUE, ...))
-	} else {
-		invisible(TRUE)
-	}
-}
-
-# overriding q, to ask via GUI instead. Arguments are not interpreted.
-"q" <- function (save = "default", status = 0, runLast = TRUE, ...) {
-	res <- .rk.do.plain.call ("quit")
-	if (length (res) && (res == "FALSE")) stop ("Quitting was cancelled")
-}
-
-"quit" <- function (save = "default", status = 0, runLast = TRUE, ...) {
-	q (save, status, runLast, ...)
-}
-
 #".rk.init.handlers" <- function () {
 #	options (warning.expression = expression ())
 #	.Internal (.addCondHands (c ("message", "warning", "error"), list (function (m) { .Call ("rk.do.condition", c ("m", conditionMessage (m))) }, function (w) { .Call ("rk.do.condition", c ("w", conditionMessage (w))) }, function (e) { .Call ("rk.do.condition", c ("e", conditionMessage (e))) }), globalenv (), NULL, TRUE))
 #}
 
 # these functions can be used to track assignments to R objects. The main interfaces are .rk.watch.symbol (k) and .rk.unwatch.symbol (k). This works by copying the symbol to a backup environment, removing it, and replacing it by an active binding to the backup location
+#' @export
 ".rk.watched.symbols" <- new.env ()
 
-# override makeActiveBinding: If active bindings are created in globalenv (), watch them properly
-.rk.makeActiveBinding.default <- base::makeActiveBinding
-"makeActiveBinding" <- function (sym, fun, env, ...) {
-	if (identical (env, globalenv ())) {
-		.rk.makeActiveBinding.default (sym, fun, .rk.watched.symbols, ...)
-		f <- .rk.make.watch.f (sym)
-		.rk.makeActiveBinding.default (sym, f, globalenv (), ...)
-	} else {
-		.rk.makeActiveBinding.default (sym, fun, env, ...)
-	}
-}
-
+#' @export
 ".rk.make.watch.f" <- function (k) {
 	# we need to make sure, the functions we use are *not* looked up as symbols in .GlobalEnv.
 	# else, for instance, if the user names a symbol "missing", and we try to resolve it in the
@@ -209,7 +207,10 @@
 	function (value) {
 		if (!missing (value)) {
 			assign (k, value, envir=.rk.watched.symbols)
-			.rk.do.call ("ws", k);
+			.Call ("rk.do.command", c ("ws", k));
+#			NOTE: the above is essentially the same a
+#				.rk.do.call ("ws", k);
+#			only minimally faster.
 			invisible (value)
 		} else {
 			get (k, envir=.rk.watched.symbols)
@@ -217,6 +218,7 @@
 	}
 }
 
+#' @export
 ".rk.watch.symbol" <- function (k) {
 	f <- .rk.make.watch.f (k)
 	.Call ("rk.copy.no.eval", k, globalenv(), .rk.watched.symbols);
@@ -229,6 +231,7 @@
 }
 
 # not needed by rkward but provided for completeness
+#' @export
 ".rk.unwatch.symbol" <- function (k) {
 	rm (list=k, envir=globalenv ())
 
@@ -239,6 +242,7 @@
 	invisible (TRUE)
 }
 
+#' @export
 ".rk.watch.globalenv" <- function () {
 	newlist <- ls (globalenv (), all.names=TRUE)
 	oldlist <- ls (.rk.watched.symbols, all.names=TRUE)
@@ -255,6 +259,7 @@
 	}
 }
 
+#' @export
 ".rk.get.vector.data" <- function (x) {
 	ret <- list ();
 	ret$data <- as.vector (unclass (x));
@@ -267,36 +272,37 @@
 }
 
 # Change storage type of x to mode newmode.
-# Most attributes will be kept, but the data is erased!
+# Keeps the .rk.meta attribute, and levels attributes, but the data is erased!
+#' @export
 ".rk.set.vector.mode" <- function (x, fun, envir=parent.frame ()) {
-	old_attr <- attributes (x)
-	old_attr$class <- NULL
-	old_attr[[".rk.invalid.fields"]] <- list ()	# will be reset, anyway!
+	y <- fun(rep(NA, length.out = length(x)))
 
-	y <- fun (rep (NA, length.out=length (x)))
+	newattrs <- attributes(y)
+	if (is.null (newattrs)) newattrs <- list ()
+	newattrs[[".rk.meta"]] <- attributes(x)[[".rk.meta"]]
+	lvls <- attributes(x)[["levels"]]
+	if (!is.null (lvls)) newattrs[["levels"]] <- lvls
+	attributes(y) <- newattrs
 
-	# merge old attributes with new ones
-	newattrs <- attributes (y)
-	for (nattr in names (newattrs)) {
-		old_attr[[nattr]] <- newattrs[[nattr]]
-	}
-
-	attributes (y) <- old_attr
 	eval (substitute (x <- y), envir=envir)
 }
 
+#' @export
 ".rk.get.structure" <- function (x, name, envlevel=0, namespacename=NULL) {
 	.Call ("rk.get.structure", x, as.character (name), as.integer (envlevel), namespacename)
 }
 
+#' @export
 ".rk.try.get.namespace" <- function (name) {
 	tryCatch (asNamespace (name), error = function(e) NULL)
 }
 
+#' @export
 ".rk.get.structure.global" <- function (name, envlevel=0, namespacename=NULL) {
 	.Call ("rk.get.structure.global", as.character (name), as.integer (envlevel), namespacename)
 }
 
+#' @export
 ".rk.get.slots" <- function (x) {
 	slotnames <- methods::slotNames (class (x))
 	ret <- lapply (slotnames, function (slotname) slot (x, slotname))
@@ -304,6 +310,7 @@
 	ret
 }
 
+#' @export
 ".rk.get.environment.children" <- function (x, envlevel=0, namespacename=NULL) {
 	ret <- list ()
 
@@ -327,69 +334,36 @@
 	ret
 }
 
-"Sys.setlocale" <- function (category = "LC_ALL", locale = "", ...) {
-	if (category == "LC_ALL" || category == "LC_CTYPE" || category == "LANG") {
-		allow <- .rk.do.plain.call ("preLocaleChange", NULL)
-		if (length (allow) && (allow == "FALSE")) stop ("Changing the locale was cancelled by user")
-
-		ret <- base::Sys.setlocale (category, locale, ...)
-
-		.Call ("rk.update.locale")
-		ret
-	} else {
-		base::Sys.setlocale (category, locale, ...)
-	}
-}
-
-"setwd" <- function () {
-	ret <- eval (body (base::setwd))
-	.rk.do.plain.call ("wdChange", base::getwd (), synchronous=FALSE)
-	invisible (ret)
-}
-formals (setwd) <- formals (base::setwd)
-
 # hidden, as this is not portable to different output formats
+#' @export
 ".rk.cat.output" <- function (x) {
 	cat (x, file = rk.get.output.html.file(), append = TRUE)
 }
 
+#' @export
 ".rk.rerun.plugin.link" <- function (plugin, settings, label) {
 	.rk.cat.output (paste ("<a href=\"rkward://runplugin/", plugin, "/", URLencode (settings), "\">", label, "</a>", sep=""))
 }
 
+#' @export
 ".rk.make.hr" <- function () {
-	.rk.cat.output ("<hr>\n");
-}
-
-# Start recording commands that are submitted from rkward to R.
-# filename: filename to write to (file will be truncated!).
-# include.all: By default, some types of command are filtered (internal synchronisation commands, and run again links). Should these be included?
-# To stop recording, supply NULL or "" as filename
-# Currently used for the purpose of automated testing, only. Perhaps in the future
-# this or a similar mechanism could also be added as a user feature.
-"rk.record.commands" <- function (filename, include.all = FALSE) {
-	if (is.null (filename)) filename = ""
-
-	res <- .rk.do.plain.call ("recordCommands", c(as.character (filename), if (include.all) "include.all" else "normal"))
-
-	if (!length (res)) invisible (TRUE)
-	else {
-		warning (res)
-		invisible (FALSE)
-	}
+	.rk.cat.output ("<hr />\n");
 }
 
 # General purpose storage environment (which will hopefully never get locked by R)
+#' @export
 ".rk.variables" <- new.env ()
-.rk.variables$.rk.active.device <- 1
-.rk.variables$.rk.output.html.file <- NULL
-.rk.variables$.rk.rkreply <- NULL
-.rk.variables$available.packages.cache <- NULL
+assign(".rk.active.device", 1, envir=.rk.variables)
+assign(".rk.output.html.file", NULL, envir=.rk.variables)
+assign(".rk.rkreply", NULL, envir=.rk.variables)
+assign("available.packages.cache", NULL, envir=.rk.variables)
 
+#' @export
 ".rk.backups" <- new.env ()
 
 # where masking is not enough, we need to assign in the original environment / namespace. This can only be done after package loading,
 # so we have a separate function for that.
+#' @export
 ".rk.fix.assignments" <- function () {
 	## History manipulation function (overloads for functions by the same name in package utils)
 	rk.replace.function ("loadhistory",  as.environment ("package:utils"),
