@@ -1,7 +1,48 @@
 ## Public functions manipulating "graphics" should be stored here.
 ## These functions are accessible to the user.
 
-# Requests a graph to be written.
+#' Plot graphics to HTML output file
+#'
+#' \code{rk.graph.on()} creates an R device that will plot to the output html page in RKWard (\url{rkward://page/rkward_output}).
+#' The default settings for \code{device.type}, \code{width}, \code{height}, and \code{quality} can be modified from Settings -> Configure RKWard -> Output.
+#'
+#' @param device.type Type of device to create / graphics format to use. Currently, supported devices are "PNG", "SVG", or "JPG".
+#'                    The default is to use the format configured in Settings -> Configure RKWard -> Output.
+#' @param width Width of graphics in pixels. The default is to use the width configured in Settings -> Configure RKWard -> Output.
+#' @param height Height of graphics in pixels. The default is to use the heigth configured in Settings -> Configure RKWard -> Output.
+#' @param quality For device.type "JPG", quality of the JPG file from 0 to 100.
+#'                The default is to use the quality configured in Settings -> Configure RKWard -> Output.
+#' @param ... Further options will be passed to the graphics device used internally.
+#'
+#' \bold{Warning}: It is advised to use \code{rk.graph.off} and \bold{not} \code{dev.off} to close the device opened by
+#' \code{rk.graph.on}. \code{dev.print(device = rk.graph.on)} is a \bold{wrong} usage for this "device," and will result in errors.
+#' 
+#' @author Thomas Friedrichsmeier \email{rkward-devel@@lists.sourceforge.net}
+#' 
+#' @seealso \link{rk.results} \link{rk.print} \link{rk.get.output.html.file} \link{dev.off} \link{svg} \link{png} \link{jpg}
+#'
+#' @examples
+#' require (rkward)
+#' 
+#' ## Plot directly to the output (html) file, by-passing screen device:
+#' rk.graph.on ("JPG", 480, 480, 75)
+#' plot (rnorm (100))
+#' rk.graph.off ()
+#' 
+#' ## Copy the displayed plot to the output:
+#' plot (rnorm (100))
+#' dev.copy (device = rk.graph.on)
+#' rk.graph.off ()
+#' 
+#' ## WRONG USAGE: not run:
+#' #plot (rnorm (100))
+#' #dev.print (device = rk.graph.on)
+#'
+#' @keywords devices
+#'
+#' @export
+#' @aliases rk.graph.on rk.graph.off
+#' @rdname rk.graph.on
 rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOption ("rk.graphics.width"), height=getOption ("rk.graphics.height"), quality, ...) 
 {
 	if (!is.numeric (width)) width <- 480
@@ -44,6 +85,10 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 	invisible (ret)
 }
 
+#' \code{rk.graph.off()} closes the device that was opened by \code{rk.graph.on}. 
+#'
+#' @rdname rk.graph.on
+#' @export
 "rk.graph.off" <- function(){
 	.rk.cat.output ("\n")	# so the output will be auto-refreshed
 	ret <- dev.off()
@@ -71,6 +116,7 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 #'   \code{\link{rk.graph.on}}
 #' @keywords utilities device
 #' @rdname rk.printer.device
+#' @export
 #' @examples
 #' 
 #' ## Not run:
@@ -86,6 +132,7 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 	.rk.variables$.rk.printer.devices[[as.character (dev.cur ())]] <- tf
 }
 
+#' @export
 "rk.duplicate.device" <- function (devId = dev.cur ())
 {
 	dev.set (devId)
@@ -93,6 +140,7 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 }
 
 # A global history of various graphics calls;
+#' @export
 "rk.record.plot" <- function ()
 {
 	env <- environment()
@@ -194,7 +242,7 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 		
 		# all open screen devices
 		.osd <- which (names (dev.list ()) %in% deviceIsInteractive ()) + 1
-		.opd <- unlist (.rk.variables$.rk.preview.devices)
+		.opd <- .rk.list.preview.device.numbers()
 		# to be managed devices:
 		if (length (.opd) > 0) .osd <-.osd [!(.osd %in% .opd)]
 		if (length (.osd) == 0) return (invisible ())
@@ -798,6 +846,7 @@ rk.record.plot <- rk.record.plot ()
 
 # Users should use only these wrappers:
 # 1 is always the null device
+#' @export
 "rk.toggle.plot.history" <- function (x = TRUE)
 {
 	if (x) {
@@ -808,56 +857,67 @@ rk.record.plot <- rk.record.plot ()
 	options ("rk.enable.graphics.history" = x)
 	invisible ()
 }
+#' @export
 "rk.first.plot" <- function (devId = dev.cur ())
 {
 	if (!getOption ("rk.enable.graphics.history")) return (invisible ())
 	rk.record.plot$showFirst (devId)
 }
+#' @export
 "rk.previous.plot" <- function (devId = dev.cur ())
 {
 	if (!getOption ("rk.enable.graphics.history")) return (invisible ())
 	rk.record.plot$showPrevious (devId)
 }
+#' @export
 "rk.next.plot" <- function (devId = dev.cur ())
 {
 	if (!getOption ("rk.enable.graphics.history")) return (invisible ())
 	rk.record.plot$showNext (devId)
 }
+#' @export
 "rk.last.plot" <- function (devId = dev.cur ())
 {
 	if (!getOption ("rk.enable.graphics.history")) return (invisible ())
 	rk.record.plot$showLast (devId)
 }
+#' @export
 "rk.goto.plot" <- function (devId = dev.cur (), index=1)
 {
 	if (!getOption ("rk.enable.graphics.history")) return (invisible ())
 	rk.record.plot$showPlot (devId, index)
 }
+#' @export
 "rk.force.append.plot" <- function (devId = dev.cur ())
 {
 	if (!getOption ("rk.enable.graphics.history")) return (invisible ())
 	rk.record.plot$forceAppend (devId)
 }
+#' @export
 "rk.removethis.plot" <- function (devId = dev.cur ())
 {
 	if (!getOption ("rk.enable.graphics.history")) return (invisible ())
 	rk.record.plot$removePlot (devId)
 }
+#' @export
 "rk.clear.plot.history" <- function ()
 {
 	if (!getOption ("rk.enable.graphics.history")) return (invisible ())
 	rk.record.plot$clearHistory ()
 }
+#' @export
 "rk.show.plot.info" <- function (devId = dev.cur ())
 {
 	if (!getOption ("rk.enable.graphics.history")) return (invisible ())
 	rk.record.plot$showPlotInfo (devId)
 }
+#' @export
 "rk.verify.plot.hist.limits" <- function (lmax)
 {
 	if (!getOption ("rk.enable.graphics.history")) return (invisible ())
 	rk.record.plot$.verify.hist.limits (as.integer (lmax))
 }
+#' @export
 "rk.plot.history.summary" <- function (which = NULL, type = c ("devices", "history"))
 {
 	ret <- NULL
