@@ -163,33 +163,37 @@ rktest.cleanRKTestSuite <- function (suite) {
 }
 
 ## Convenience functions for replacing / restoring functions for the test runs
-rktest.replace <- function (name, replacement, envir=as.environment ("package:rkward"), backup.name=name) {
-	if (exists (backup.name, envir=.rktest.tmp.storage, inherits=FALSE)) {
+rktest.replace <- function (name, replacement, envir=as.environment ("package:rkward"), backup.name=name, targetEnv=.rktest.tmp.storage) {
+	if (exists (backup.name, envir=targetEnv, inherits=FALSE)) {
 		message ("It looks like ", name, " has already been replaced. Not replacing it again.")
 	} else {
-# 		# Apparently R 2.14.x starts forcing namespaces for all packages, which makes things a bit more difficult
 		if (identical (envir, as.environment ("package:rkward"))) {
-			try (rktest.replace (name, replacement, asNamespace ("rkward"), backup.name), silent=TRUE)
+# 		# Apparently R 2.14.x starts forcing namespaces for all packages, which makes things a bit more difficult
+			try (unlockBinding (name, asNamespace ("rkward")), silent=TRUE)
+			try (assign (name, replacement, asNamespace ("rkward")), silent=TRUE)
+			assign (backup.name, get (name, asNamespace ("rkward")), envir=targetEnv)
+		} else {
+			assign (backup.name, get (name, envir), envir=targetEnv)
 		}
 
-		assign (backup.name, get (name, envir), envir=.rktest.tmp.storage)
-
 		environment (replacement) <- envir
-		try (unlockBinding (name, envir))
-		assign (name, replacement, envir)
+		try (unlockBinding (name, envir), silent=TRUE)
+		try (assign (name, replacement, envir), silent=TRUE)
 	}
+	return(invisible(NULL))
 }
 
-rktest.restore <- function (name, envir=as.environment ("package:rkward"), backup.name=name) {
-	if (exists (backup.name, envir=.rktest.tmp.storage, inherits=FALSE)) {
-		assign (name, get (backup.name, envir=.rktest.tmp.storage), envir=envir)
+rktest.restore <- function (name, envir=as.environment ("package:rkward"), backup.name=name, targetEnv=.rktest.tmp.storage) {
+	if (exists (backup.name, envir=targetEnv, inherits=FALSE)) {
 		if (identical (envir, as.environment ("package:rkward"))) {
-			try (assign (name, get (backup.name, envir=.rktest.tmp.storage), envir=asNamespace ("rkward")), silent=TRUE)
+			assign (name, get (backup.name, envir=targetEnv), envir=asNamespace ("rkward"))
 		}
+		try (assign (name, get (backup.name, envir=targetEnv), envir=envir), silent=TRUE)
+		rm (list=backup.name, envir=targetEnv)
 	} else {
 		message ("No backup available for ", name, ". Already restored?")
 	}
-	rm (list=backup.name, envir=.rktest.tmp.storage)
+	return(invisible(NULL))
 }
 
 ## Initialize test environment
@@ -208,12 +212,18 @@ rktest.initializeEnvironment <- function () {
 	# HACK: Override date, so we don't get a difference for each call of rk.header ()
 	rktest.replace (".rk.date", function () return ("DATE"))
 
+	# disable toc printing
+	options (.rk.suppress.toc=TRUE)
+
 	# numerical precision is often a problem. To work around this in many places, reduce default printed precision to 5 digits
 	options (digits=5)
 
 	# Make sure i18n does not get in the way
 	invisible (Sys.setenv (LANGUAGE="C"))
-	if (.Platform$OS.type == "unix") invisible (Sys.setlocale ("LC_MESSAGES", "C"))
+	invisible (Sys.setenv (LANG="C"))
+	if (.Platform$OS.type == "unix"){
+		invisible (Sys.setlocale (category="LC_MESSAGES", locale="C"))
+	}
 	options (useFancyQuotes=FALSE)
 
 	# This version of rk.set.output.html.file does not notify the frontend of the change. Without this, you'll get lots of output windows.
@@ -233,6 +243,7 @@ rktest.resetEnvironment <- function () {
 	rktest.restore ("rk.get.tempfile.name")
 	rktest.restore ("rk.set.output.html.file")
 	rktest.restore (".rk.date")
+	options (.rk.supress.toc=FALSE)
 	assign("initialized", FALSE, envir=.rktest.tmp.storage)
 }
 
