@@ -62,7 +62,7 @@
 // static
 RKConsole* RKConsole::main_console = 0;
 
-RKConsole::RKConsole (QWidget *parent, bool tool_window, const char *name) : RKMDIWindow (parent, RKMDIWindow::ConsoleWindow, tool_window, name) {
+RKConsole::RKConsole (QWidget *parent, bool tool_window, const char *name) : RKMDIWindow (parent, RKMDIWindow::ConsoleWindow, tool_window, name), commands_history (false, false) {
 	RK_TRACE (APP);
 
 	QVBoxLayout *layout = new QVBoxLayout (this);
@@ -84,6 +84,9 @@ RKConsole::RKConsole (QWidget *parent, bool tool_window, const char *name) : RKM
 	RK_ASSERT (view);
 	confint->setConfigValue ("dynamic-word-wrap", false);
 
+	KTextEditor::CodeCompletionInterface *iface = qobject_cast<KTextEditor::CodeCompletionInterface*> (view);
+	if (iface) iface->setAutomaticInvocationEnabled (false);
+
 	setFocusProxy (view);
 	setFocusPolicy (Qt::StrongFocus);
 	
@@ -131,8 +134,7 @@ RKConsole::RKConsole (QWidget *parent, bool tool_window, const char *name) : RKM
 	clear ();
 	RKCommandHighlighter::setHighlighting (doc, RKCommandHighlighter::RInteractiveSession);
 
-	commands_history = RKSettingsModuleConsole::loadCommandHistory ();
-	commands_history_position = commands_history.constEnd ();
+	commands_history.setHistory (RKSettingsModuleConsole::loadCommandHistory ());
 
 	current_command = 0;
 	current_command_displayed_up_to = 0;
@@ -148,7 +150,7 @@ RKConsole::~RKConsole () {
 	RK_TRACE (APP);
 
 	delete hinter;
-	RKSettingsModuleConsole::saveCommandHistory (commands_history);
+	RKSettingsModuleConsole::saveCommandHistory (commands_history.getHistory ());
 }
 
 QAction* RKConsole::addProxyAction (const QString& actionName, const QString& label) {
@@ -207,29 +209,23 @@ void RKConsole::setCursorClear (int line, int col) {
 bool RKConsole::handleKeyPress (QKeyEvent *e) {
 	KTextEditor::Cursor c = view->cursorPosition ();
 	int para=c.line (); int pos=c.column ();
-
-	if (para < doc->lines () - 1 || pos < prefix.length ()) {	// not inside the last line?
-		int key = e->key ();
-		if ((key != Qt::Key_Shift) && (key != Qt::Key_Control) && (key != Qt::Key_Meta) && (key != Qt::Key_Alt)) {
-			int t = (int) pos;					// adjust position before interpreting non-modifier keystroke
-			if (prefix.length()>pos) t=prefix.length ();
-			view->setCursorPosition (KTextEditor::Cursor (doc->lines () -1, t));
-		}
-	}
-
+	int key = e->key ();
+	bool is_modifier_key = ((key == Qt::Key_Shift) || (key == Qt::Key_Control) || (key == Qt::Key_Meta) || (key == Qt::Key_Alt));
+	
 	if (view->selection ()) {
 		KTextEditor::Range selrange = view->selectionRange ();
 		if (selrange.start ().line () < (doc->lines () - 1) || selrange.start ().column () < prefix.length ()) {
 			// There is a selection outside the command line
-			// Eat the key and beep (unless it's just a modifier key). Otherwise it might overwrite or delete the selection
-			if (e->key () - (e->key () & (Qt::ShiftModifier | Qt::ControlModifier | Qt::AltModifier | Qt::MetaModifier | Qt::KeypadModifier | Qt::GroupSwitchModifier))) {
-				KApplication::kApplication ()->beep ();
-				e->ignore ();
-			}
-			return true;
+			// If it's a modifier key, that's quite ok. Otherwise, we need to take off the selection, then proceed to handling the key press
+			if (is_modifier_key) return true;
+			view->setSelection (KTextEditor::Range ());
 		}
 	}
 
+	if (para < doc->lines () - 1 || pos < prefix.length ()) {	// not inside the last line?
+		if (!is_modifier_key) cursorAtTheEnd ();	// adjust position before interpreting non-modifier keystroke
+	}
+
 	if (current_command) {
 		e->ignore ();
 		return true;
@@ -237,18 +233,31 @@ bool RKConsole::handleKeyPress (QKeyEvent *e) {
 
 	previous_chunk_was_piped = false;
 
-	if (e->key () == Qt::Key_Up) {
-		commandsListUp (RKSettingsModuleConsole::shouldDoHistoryContextSensitive (e->modifiers ()));
-		return true;
-	}
-	else if (e->key () == Qt::Key_Down) {
-		commandsListDown (RKSettingsModuleConsole::shouldDoHistoryContextSensitive (e->modifiers ()));
-		return true;
-	}
-	command_edited = true; // all other keys are considered as "editing" the current comand
+	// Apparently, on MacOSX, additional modifiers may sometimes be present (KeypadModifier?), which we want to ignore.
+	const int modifier_mask = Qt::ShiftModifier | Qt::ControlModifier | Qt::AltModifier;
+	Qt::KeyboardModifiers modifier = e->modifiers () & modifier_mask;
 
-	if (e->key () == Qt::Key_Home) {
-		if (e->modifiers () == Qt::ShiftModifier) {
+	if ((key == Qt::Key_Up) || (key == Qt::Key_Down)) {
+		KTextEditor::CodeCompletionInterface *iface = qobject_cast<KTextEditor::CodeCompletionInterface*> (view);
+		if (iface && iface->isCompletionActive ()) {
+			if (key == Qt::Key_Up) triggerEditAction ("move_line_up");
+			else triggerEditAction ("move_line_down");
+			return true;
+		}
+
+		// showing completion list during history navigation is not a good idea, since it uses the same keys
+		bool auto_inv = (iface && iface->isAutomaticInvocationEnabled ());
+		if (iface) iface->setAutomaticInvocationEnabled (false);
+
+		if (key == Qt::Key_Up) commandsListUp (RKSettingsModuleConsole::shouldDoHistoryContextSensitive (modifier));
+		else commandsListDown (RKSettingsModuleConsole::shouldDoHistoryContextSensitive (modifier));
+
+		if (iface) iface->setAutomaticInvocationEnabled (auto_inv);
+		return true;
+	}
+
+	if (key == Qt::Key_Home) {
+		if (modifier == Qt::ShiftModifier) {
 			int lastline = doc->lines () - 1;
 			int firstcol = prefix.length ();
 			KTextEditor::Range newrange (lastline, firstcol, lastline, pos);
@@ -265,8 +274,8 @@ bool RKConsole::handleKeyPress (QKeyEvent *e) {
 		}
 		cursorAtTheBeginning ();
 		return true;
-	} else if (e->key () == Qt::Key_End) {
-		if (e->modifiers () == Qt::ShiftModifier) {
+	} else if (key == Qt::Key_End) {
+		if (modifier == Qt::ShiftModifier) {
 			int lastline = doc->lines () - 1;
 			int lastcol = doc->lineLength (lastline);
 			KTextEditor::Range newrange (lastline, pos, lastline, lastcol);
@@ -283,32 +292,39 @@ bool RKConsole::handleKeyPress (QKeyEvent *e) {
 		}
 		cursorAtTheEnd ();
 		return true;
-	} else if (e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return) {
+	} else if (key == Qt::Key_Enter || key == Qt::Key_Return) {
+		// currently, these are auto-text hints, only
+		KTextEditor::CodeCompletionInterface *iface = qobject_cast<KTextEditor::CodeCompletionInterface*> (view);
+		if (iface && iface->isCompletionActive ()) {
+			iface->forceCompletion ();
+			return true;
+		}
+
 		hinter->hideArgHint ();
-		addCommandToHistory (currentEditingLine ());
+		commands_history.append (currentEditingLine ());
 		submitCommand ();
 		return true;
-	} else if (e->key () == Qt::Key_Left){
+	} else if (key == Qt::Key_Left){
 		if (pos <= prefix.length ()) return true;
 
-		if (e->modifiers () == Qt::NoModifier) setCursorClear (para, pos - 1);
-		else if (e->modifiers () == Qt::ShiftModifier) triggerEditAction ("select_char_left");
-		else if (e->modifiers () == Qt::ControlModifier) triggerEditAction ("word_left");
-		else if (e->modifiers () == (Qt::ControlModifier + Qt::ShiftModifier)) triggerEditAction ("select_word_left");
+		if (modifier == Qt::NoModifier) setCursorClear (para, pos - 1);
+		else if (modifier == Qt::ShiftModifier) triggerEditAction ("select_char_left");
+		else if (modifier == Qt::ControlModifier) triggerEditAction ("word_left");
+		else if (modifier == (Qt::ControlModifier + Qt::ShiftModifier)) triggerEditAction ("select_word_left");
 		else return false;
 
 		return true;
-	} else if (e->key () == Qt::Key_Right){
+	} else if (key == Qt::Key_Right){
 		if (pos >= doc->lineLength (para)) return true;
 
-		if (e->modifiers () == Qt::NoModifier) setCursorClear (para, pos + 1);
-		else if (e->modifiers () == Qt::ShiftModifier) triggerEditAction ("select_char_right");
-		else if (e->modifiers () == Qt::ControlModifier) triggerEditAction ("word_right");
-		else if (e->modifiers () == (Qt::ControlModifier + Qt::ShiftModifier)) triggerEditAction ("select_word_right");
+		if (modifier == Qt::NoModifier) setCursorClear (para, pos + 1);
+		else if (modifier == Qt::ShiftModifier) triggerEditAction ("select_char_right");
+		else if (modifier == Qt::ControlModifier) triggerEditAction ("word_right");
+		else if (modifier == (Qt::ControlModifier + Qt::ShiftModifier)) triggerEditAction ("select_word_right");
 		else return false;
 
 		return true;
-	} else if (e->key () == Qt::Key_Backspace){
+	} else if (key == Qt::Key_Backspace){
 		if(pos<=prefix.length ()) return true;
 
 		if (view->selection ()) {
@@ -318,7 +334,7 @@ bool RKConsole::handleKeyPress (QKeyEvent *e) {
 			doc->removeText (KTextEditor::Range (para, pos, para, pos-1));
 		}
 		return true;
-	} else if (e->key () == Qt::Key_Delete) {
+	} else if (key == Qt::Key_Delete) {
 		// not currently handled by an action in kateview internal, but better not rely on that fact!
 		if (view->selection ()) {
 			doc->removeText (view->selectionRange ());
@@ -327,7 +343,11 @@ bool RKConsole::handleKeyPress (QKeyEvent *e) {
 			doc->removeText (KTextEditor::Range (para, pos, para, pos + 1));
 		}
 		return true;
-	} else if (e->key () == Qt::Key_Tab) {
+	} else if (key == Qt::Key_Tab) {
+		KTextEditor::CodeCompletionInterface *iface = qobject_cast<KTextEditor::CodeCompletionInterface*> (view);
+		if (iface && iface->isCompletionActive ()) {
+			return false;
+		}
 		doTabCompletion ();
 		return true;
 	}
@@ -455,7 +475,7 @@ bool RKConsole::eventFilter (QObject *o, QEvent *e) {
 
 	if (e->type () == QEvent::KeyPress) {
 		QKeyEvent *k = (QKeyEvent *)e;
-		return handleKeyPress (k);
+		return (handleKeyPress (k));
 	} else if (e->type () == QEvent::MouseButtonPress) {
 		// we seem to need this, as the kateview will swallow the contextMenuEvent, otherwise
 		QMouseEvent *m = (QMouseEvent *)e;
@@ -555,64 +575,17 @@ void RKConsole::submitCommand () {
 void RKConsole::commandsListUp (bool context_sensitive) {
 	RK_TRACE (APP);
 
-	// if we are at the last line, i.e. not yet navigating the command history, store the current command
-	if (commands_history.constEnd () == commands_history_position) history_editing_line = currentEditingLine ();
-
-	if (context_sensitive) {
-		if (command_edited) {
-			command_history_context = currentEditingLine ();
-			commands_history_position = commands_history.constEnd ();
-			command_edited = false;
-		}
-	} else {
-		command_edited = true;
-	}
-
-	bool found = false;
-	QStringList::const_iterator it = commands_history_position;
-	while (it != commands_history.constBegin ()) {
-		--it;
-	  	if ((!context_sensitive) || (*it).startsWith (command_history_context)) { // we found a match or previous line
-			found = true;
-			break;
-		}
-	}
-
-	if (found) {		// if we did not find a previous matching line, do not touch the commands_history_position
-		commands_history_position = it;
-		setCurrentEditingLine (*commands_history_position);
-	} else {
-		KApplication::kApplication ()->beep ();
-	}
+	bool found = commands_history.up (context_sensitive, currentEditingLine ());
+	if (found) setCurrentEditingLine (commands_history.current ());
+	else KApplication::kApplication ()->beep ();
 }
 
 void RKConsole::commandsListDown (bool context_sensitive) {
 	RK_TRACE (APP);
 
-	if (context_sensitive) {
-		if (command_edited) {
-			command_history_context = currentEditingLine ();
-	  		commands_history_position = commands_history.constEnd ();
-	  		command_edited = false;
-	  		return; // back at bottommost item
-		}
-	} else {
-		command_edited = true;
-	}
-
-	if (commands_history.constEnd () == commands_history_position) {		// already at bottommost item
-		KApplication::kApplication ()->beep ();
-		return;
-	}
-
-	while (++commands_history_position != commands_history.constEnd ()) {
-		if ((!context_sensitive) || (*commands_history_position).startsWith (command_history_context)) { // we found a match or next line
-			break;
-		}
-	}
-
-	if (commands_history.constEnd () == commands_history_position) setCurrentEditingLine (history_editing_line);
-	else setCurrentEditingLine (*commands_history_position);
+	bool found = commands_history.down (context_sensitive, currentEditingLine ());
+	if (found) setCurrentEditingLine (commands_history.current ());
+	else KApplication::kApplication ()->beep ();
 }
 
 void RKConsole::rCommandDone (RCommand *command) {
@@ -632,7 +605,7 @@ void RKConsole::rCommandDone (RCommand *command) {
 		incomplete_command.clear ();
 	}
 
-	commands_history_position = commands_history.constEnd ();
+	commands_history.goToEnd ();
 	showPrompt ();
 	tryNextInBuffer ();
 }
@@ -641,7 +614,26 @@ void RKConsole::newOutput (RCommand *, ROutput *output) {
 	RK_TRACE (APP);
 
 	int start_line = doc->lines () -1;
-	doc->insertText (doc->documentEnd (), output->output);
+
+	// split by and handle carriage returns
+	const QString outstr = output->output;
+	int start_pos = 0;
+	int end_pos = outstr.size () - 1;
+	QChar c;
+	for (int pos = 0; pos <= end_pos; ++pos) {
+		c = output->output.at (pos);
+		if (c == '\r') {
+			/* NOTE: My first approach was to split the whole string by newlines, and insert each line separately. This allowed for a minor
+			 * optimization when hitting a carriage return (the string before the '\r' could simply be ignored, then), however it caused
+			 * around 10% slowdown when printing large amounts of output.
+			 * Thus, instead, when hitting a CR, we first insert everything before that into the document, then reset the line. */
+			doc->insertText (doc->documentEnd (), outstr.mid (start_pos, pos - start_pos));
+			doc->removeLine (doc->lines () - 1);
+			doc->insertLine (doc->lines (), QString ());
+			start_pos = pos + 1;
+		}
+	}
+	if (start_pos <= end_pos) doc->insertText (doc->documentEnd (), outstr.mid (start_pos, end_pos - start_pos + 1));
 
 	int end_line = doc->lines () -1;
 	if (output->type != ROutput::Output) {
@@ -752,31 +744,10 @@ void RKConsole::clear () {
 	showPrompt ();
 }
 
-void RKConsole::addCommandToHistory (const QString &command) {
-	RK_TRACE (APP);
-	if ((!command.isEmpty ()) && (commands_history.isEmpty () || commands_history.last() != command)) { // don't add empty or duplicate lines
-		commands_history.append (command);
-	}
-
-	if (RKSettingsModuleConsole::maxHistoryLength ()) {
-		uint c = commands_history.count ();
-		for (uint ui = c; ui > RKSettingsModuleConsole::maxHistoryLength (); --ui) {
-			commands_history.pop_front ();
-		}
-	}
-
-	history_editing_line = QString ();
-	commands_history_position = commands_history.constEnd ();
-	command_edited = false;
-}
-
 void RKConsole::setCommandHistory (const QStringList &new_history, bool append) {
 	RK_TRACE (APP);
 
-	if (append) commands_history.append (new_history);
-	else commands_history = new_history;
-
-	addCommandToHistory (QString ());	// side-effect of checking history length
+	commands_history.setHistory (new_history, append);
 }
 
 void RKConsole::userLoadHistory (const KUrl &_url) {
@@ -970,7 +941,7 @@ void RKConsole::pipeCommandThroughConsoleLocal (const QString &command_string) {
 		QStringList lines = command_string.split ('\n', QString::SkipEmptyParts);
 		if ((RKSettingsModuleConsole::addPipedCommandsToHistory() == RKSettingsModuleConsole::AlwaysAdd) || (lines.count () == 1)) {
 			for (int i = 0; i < lines.count (); ++i) {
-				addCommandToHistory (lines[i]);
+				commands_history.append (lines[i]);
 			}
 		}
 	}
