@@ -82,6 +82,8 @@
 #include "windows/rktoplevelwindowgui.h"
 #include "windows/rkfilebrowser.h"
 #include "windows/rktoolwindowlist.h"
+#include "windows/rkdebugconsole.h"
+#include "windows/rkcallstackviewer.h"
 #include "rkconsole.h"
 #include "debug.h"
 #include "version.h"
@@ -131,6 +133,7 @@ RKWardMainWindow::RKWardMainWindow (RKWardStartupOptions *options) : KParts::Mai
 	RKWorkplace::mainWorkplace ()->initActions (actionCollection (), "left_window", "right_window");
 	setCentralWidget (RKWorkplace::mainWorkplace ());
 	connect (RKWorkplace::mainWorkplace ()->view (), SIGNAL (captionChanged (const QString &)), this, SLOT (setCaption (const QString &)));
+	connect (RKWorkplace::mainWorkplace (), SIGNAL (workspaceUrlChanged(const KUrl&)), this, SLOT (addWorkspaceUrl(const KUrl&)));
 
 	part_manager = new KParts::PartManager (this);
 	// When the manager says the active part changes,
@@ -233,7 +236,7 @@ void RKWardMainWindow::doPostInit () {
 			openWorkspace (result.open_url);
 		} else {
 			if (result.result == StartupDialog::ChoseFile) {
-				slotFileOpenWorkspace ();
+				fileOpenNoSave (KUrl());
 			} else if (result.result == StartupDialog::EmptyTable) {
 				RKWorkplace::mainWorkplace ()->editNewDataFrame (i18n ("my.data"));
 			}
@@ -285,7 +288,6 @@ void RKWardMainWindow::startR () {
 
 	RKGlobals::rinter = new RInterface ();
 	new RObjectList ();
-	connect (RObjectList::getObjectList (), SIGNAL (workspaceUrlChanged(const KUrl&)), this, SLOT (addWorkspaceUrl(const KUrl&)));
 
 	RObjectBrowser::mainBrowser ()->unlock ();
 }
@@ -343,6 +345,15 @@ void RKWardMainWindow::initToolViewsAndR () {
 	RKHelpSearchWindow::main_help_search = help_search;
 	RKToolWindowList::registerToolWindow (help_search, "helpsearch", RKToolWindowList::Bottom, Qt::AltModifier + Qt::Key_6);
 
+	RKCallstackViewer::_instance = new RKCallstackViewer (0, true);
+	RKCallstackViewer::instance ()->setCaption (i18n ("Debugger Frames"));
+	RKToolWindowList::registerToolWindow (RKCallstackViewer::instance (), "debugframes", RKToolWindowList::Right, Qt::AltModifier + Qt::Key_8);
+
+	// HACK: Creating this _after_ the callstackviewer is important, so the debug console will end up the active window when entering a debug context
+	RKDebugConsole::_instance = new RKDebugConsole (0, true);
+	RKDebugConsole::instance ()->setCaption (i18n ("Debugger Console"));
+	RKToolWindowList::registerToolWindow (RKDebugConsole::instance (), "debugconsole", RKToolWindowList::Nowhere, Qt::AltModifier + Qt::Key_7);
+
 	RKWorkplace::mainWorkplace ()->placeToolWindows ();
 }
 
@@ -407,11 +418,11 @@ void RKWardMainWindow::initActions() {
 
 	// These two currently do the same thing
 	action = actionCollection ()->addAction ("load_unload_libs", this, SLOT (slotFileLoadLibs()));
-	action->setText (i18n ("Load / Unload Packages"));
+	action->setText (i18n ("Manage R packages..."));
 	action->setIcon (RKStandardIcons::getIcon (RKStandardIcons::ActionConfigurePackages));
 
 	action = actionCollection ()->addAction ("configure_packages", this, SLOT (slotFileLoadLibs()));
-	action->setText (i18n ("Configure Packages"));
+	action->setText (i18n ("Manage R packages..."));
 	action->setIcon (RKStandardIcons::getIcon (RKStandardIcons::ActionConfigurePackages));
 
 	setStandardToolBarMenuEnabled (true);
@@ -430,8 +441,7 @@ void RKWardMainWindow::initActions() {
 	window_detach->setIcon (RKStandardIcons::getIcon (RKStandardIcons::ActionDetachWindow));
 	window_detach->setText (i18n ("Detach"));
 
-	configure = actionCollection ()->addAction ("configure", this, SLOT (slotConfigure()));
-	configure->setText (i18n ("Configure RKWard"));
+	configure = actionCollection ()->addAction (KStandardAction::Preferences, "options_configure", this, SLOT (slotConfigure()));
 
 	edit_menu_dummy = actionCollection ()->addAction ("edit_menu_dummy", this);
 	edit_menu_dummy->setText (i18n ("[No actions available for current view]"));
@@ -650,7 +660,7 @@ bool RKWardMainWindow::doQueryQuit () {
 	int res;
 	res = KMessageBox::questionYesNoCancel (this, i18n ("Quitting RKWard: Do you want to save the workspace?"), i18n ("Save Workspace?"), KStandardGuiItem::save (), KStandardGuiItem::discard (), KGuiItem (i18n ("Don't quit")));
 	if (res == KMessageBox::Yes) {
-		new RKSaveAgent (RObjectList::getObjectList ()->getWorkspaceURL (), false, RKSaveAgent::DoNothing);
+		new RKSaveAgent (RKWorkplace::mainWorkplace ()->workspaceURL (), false, RKSaveAgent::DoNothing);
 	} else if (res == KMessageBox::Cancel) {
 		slotSetStatusReady ();
 		return false;
@@ -694,9 +704,9 @@ void RKWardMainWindow::fileOpenNoSave (const KUrl &url) {
 #ifdef Q_WS_WIN
 	// getOpenUrl(KUrl("kfiledialog:///<rfiles>"), ...) causes a hang on windows (KDElibs 4.2.3).
 #	warning Track this bug down and/or report it
-		lurl = KFileDialog::getOpenUrl (KUrl (), i18n("*|All files"), this, i18n("Open File..."));
+		lurl = KFileDialog::getOpenUrl (KUrl (), i18n("%1|R Workspace Files (%1)\n*|All files", RKSettingsModuleGeneral::workspaceFilenameFilter ()), this, i18n("Select workspace to open..."));
 #else
-		lurl = KFileDialog::getOpenUrl (KUrl ("kfiledialog:///<rfiles>"), i18n("*|All files"), this, i18n("Open File..."));
+		lurl = KFileDialog::getOpenUrl (KUrl ("kfiledialog:///<rfiles>"), i18n("%1|R Workspace Files (%1)\n*|All files", RKSettingsModuleGeneral::workspaceFilenameFilter ()), this, i18n("Select workspace to open..."));
 #endif
 	}
 	if (!lurl.isEmpty ()) {
@@ -717,7 +727,7 @@ void RKWardMainWindow::fileOpenAskSave (const KUrl &url) {
 	if (res == KMessageBox::No) {
 		fileOpenNoSave (url);
 	} else if (res == KMessageBox::Yes) {
-		new RKSaveAgent (RObjectList::getObjectList ()->getWorkspaceURL (), false, RKSaveAgent::Load, url);
+		new RKSaveAgent (RKWorkplace::mainWorkplace ()->workspaceURL (), false, RKSaveAgent::Load, url);
 	}
 	// else: cancel. Don't do anything
 }
@@ -741,12 +751,12 @@ void RKWardMainWindow::slotFileLoadLibs () {
 
 void RKWardMainWindow::slotFileSaveWorkspace () {
 	RK_TRACE (APP);
-	new RKSaveAgent (RObjectList::getObjectList ()->getWorkspaceURL ());
+	new RKSaveAgent (RKWorkplace::mainWorkplace ()->workspaceURL ());
 }
 
 void RKWardMainWindow::slotFileSaveWorkspaceAs () {
 	RK_TRACE (APP);
-	new RKSaveAgent (RObjectList::getObjectList ()->getWorkspaceURL (), true);
+	new RKSaveAgent (RKWorkplace::mainWorkplace ()->workspaceURL (), true);
 }
 
 void RKWardMainWindow::addWorkspaceUrl (const KUrl &url) {
@@ -871,8 +881,8 @@ void RKWardMainWindow::openHTML (const KUrl &url) {
 void RKWardMainWindow::setCaption (const QString &) {
 	RK_TRACE (APP);
 
-	QString wcaption = RObjectList::getObjectList ()->getWorkspaceURL ().fileName ();
-	if (wcaption.isEmpty ()) wcaption = RObjectList::getObjectList ()->getWorkspaceURL ().prettyUrl ();
+	QString wcaption = RKWorkplace::mainWorkplace ()->workspaceURL ().fileName ();
+	if (wcaption.isEmpty ()) wcaption = RKWorkplace::mainWorkplace ()->workspaceURL ().prettyUrl ();
 	if (wcaption.isEmpty ()) wcaption = i18n ("[Unnamed Workspace]");
 	RKMDIWindow *window = RKWorkplace::mainWorkplace ()->view ()->activePage ();
 	if (window) wcaption.append (" - " + window->fullCaption ());
