@@ -2,7 +2,7 @@
                           rkcommandeditorwindow  -  description
                              -------------------
     begin                : Mon Aug 30 2004
-    copyright            : (C) 2004, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2009, 2010, 2011, 2012 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -18,10 +18,10 @@
 
 #include <kxmlguifactory.h>
 
-#include <ktexteditor/configinterface.h>
-#include <ktexteditor/sessionconfiginterface.h>
 #include <ktexteditor/editorchooser.h>
 #include <ktexteditor/modificationinterface.h>
+#include <ktexteditor/markinterface.h>
+#include <ktexteditor/sessionconfiginterface.h>
 
 #include <qlayout.h>
 #include <qapplication.h>
@@ -63,6 +63,7 @@
 #include "../rkglobals.h"
 #include "../rkward.h"
 #include "rkhelpsearchwindow.h"
+#include "rkworkplace.h"
 
 #include "../debug.h"
 
@@ -96,6 +97,7 @@ RKCommandEditorWindow::RKCommandEditorWindow (QWidget *parent, bool use_r_highli
 	if (em_iface) em_iface->setModifiedOnDiskWarning (true);
 	else RK_ASSERT (false);
 	m_view = m_doc->createView (this);
+	m_doc->editor ()->readConfig ();
 
 	setFocusProxy (m_view);
 	setFocusPolicy (Qt::StrongFocus);
@@ -157,6 +159,24 @@ RKCommandEditorWindow::RKCommandEditorWindow (QWidget *parent, bool use_r_highli
 
 RKCommandEditorWindow::~RKCommandEditorWindow () {
 	RK_TRACE (COMMANDEDITOR);
+
+	// NOTE: TODO: Ideally we'd only write out a changed config, but how to detect config changes?
+	// 	Alternatively, only for the last closed script window
+	m_doc->editor ()->writeConfig ();
+	if (!url ().isEmpty ()) {
+		KTextEditor::SessionConfigInterface *iface = qobject_cast<KTextEditor::SessionConfigInterface*> (m_doc);
+		QString p_url = RKWorkplace::mainWorkplace ()->portableUrl (m_doc->url ());
+		if (iface) {
+			KConfigGroup conf (RKWorkplace::mainWorkplace ()->workspaceConfig (), QString ("SkriptDocumentSettings %1").arg (p_url));
+			iface->writeSessionConfig (conf);
+		}
+		iface = qobject_cast<KTextEditor::SessionConfigInterface*> (m_view);
+		if (iface) {
+			KConfigGroup conf (RKWorkplace::mainWorkplace ()->workspaceConfig (), QString ("SkriptViewSettings %1").arg (p_url));
+			iface->writeSessionConfig (conf);
+		}
+	}
+
 	delete hinter;
 	delete m_doc;
 	if (!delete_on_close.isEmpty ()) KIO::del (delete_on_close)->start ();
@@ -342,12 +362,34 @@ void RKCommandEditorWindow::setReadOnly (bool ro) {
 	m_doc->setReadWrite (!ro);
 }
 
-bool RKCommandEditorWindow::openURL (const KUrl &url, const QString& encoding, bool use_r_highlighting, bool read_only, bool delete_on_close){
+bool RKCommandEditorWindow::openURL (const KUrl url, const QString& encoding, bool use_r_highlighting, bool read_only, bool delete_on_close){
 	RK_TRACE (COMMANDEDITOR);
 
 	// encoding must be set *before* loading the file
 	if (!encoding.isEmpty ()) m_doc->setEncoding (encoding);
-	if (m_doc->openUrl (url)){
+	if (m_doc->openUrl (url)) {
+		if (!delete_on_close) {	// don't litter config with temporary files
+			KTextEditor::SessionConfigInterface *iface = qobject_cast<KTextEditor::SessionConfigInterface*> (m_doc);
+			QString p_url = RKWorkplace::mainWorkplace ()->portableUrl (m_doc->url ());
+			if (iface) {
+				KConfigGroup conf (RKWorkplace::mainWorkplace ()->workspaceConfig (), QString ("SkriptDocumentSettings %1").arg (p_url));
+				// HACK: Hmm. KTextEditor::Document's readSessionConfig() simply restores too much. Yes, I want to load bookmarks and stuff.
+				// I do not want to mess with encoding, or risk loading a different url, after the doc is already loaded!
+				if (!encoding.isEmpty () && (conf.readEntry ("Encoding", encoding) != encoding)) conf.writeEntry ("Encoding", encoding);
+				if (conf.readEntry ("URL", url) != url) conf.writeEntry ("URL", url);
+				/* HACK: What the...?! Somehow, at least on longer R scripts, stored Mode="Normal" in combination with R Highlighting
+				 * causes code folding to fail (KDE 4.8.4, https://sourceforge.net/tracker/?func=detail&atid=459007&aid=3564769&group_id=50231).
+				 * Forcing Mode == Highlighting appears to help. */
+				conf.writeEntry ("Mode", conf.readEntry ("Highlighting", "Normal"));
+				iface->readSessionConfig (conf);
+			}
+			iface = qobject_cast<KTextEditor::SessionConfigInterface*> (m_view);
+			if (iface) {
+				KConfigGroup conf (RKWorkplace::mainWorkplace ()->workspaceConfig (), QString ("SkriptViewSettings %1").arg (p_url));
+				iface->readSessionConfig (conf);
+			}
+		}
+
 		if (use_r_highlighting) RKCommandHighlighter::setHighlighting (m_doc, RKCommandHighlighter::RScript);
 		setReadOnly (read_only);
 
@@ -358,7 +400,7 @@ bool RKCommandEditorWindow::openURL (const KUrl &url, const QString& encoding, b
 				RK_ASSERT (false);
 				return true;
 			}
-			RKCommandEditorWindow::delete_on_close=url;
+			RKCommandEditorWindow::delete_on_close = url;
 		}
 
 		return true;
@@ -481,9 +523,46 @@ void RKCommandEditorWindow::insertText (const QString &text) {
 	setFocus();
 }
 
+void RKCommandEditorWindow::restoreScrollPosition () {
+	RK_TRACE (COMMANDEDITOR);
+
+	KTextEditor::Cursor c = saved_scroll_position;
+	c.setLine (qMin (c.line (), m_doc->lines () - 1));
+	if (c.column () >= m_doc->lineLength (c.line ())) c.setColumn (0);
+	m_view->setCursorPosition (c);
+}
+
+void RKCommandEditorWindow::saveScrollPosition () {
+	RK_TRACE (COMMANDEDITOR);
+
+	KTextEditor::Cursor c = m_view->cursorPosition ();
+	if (!c.isValid ()) c = KTextEditor::Cursor::start ();
+	saved_scroll_position = c;
+}
+
 void RKCommandEditorWindow::setText (const QString &text) {
 	RK_TRACE (COMMANDEDITOR);
+	bool old_rw = m_doc->isReadWrite ();
+	if (!old_rw) m_doc->setReadWrite (true);
 	m_doc->setText (text);
+	KTextEditor::MarkInterface *markiface = qobject_cast<KTextEditor::MarkInterface*> (m_doc);
+	if (markiface) markiface->clearMarks ();
+	if (!old_rw) m_doc->setReadWrite (false);
+}
+
+void RKCommandEditorWindow::highlightLine (int linenum) {
+	RK_TRACE (COMMANDEDITOR);
+
+	KTextEditor::MarkInterface *markiface = qobject_cast<KTextEditor::MarkInterface*> (m_doc);
+	if (!markiface) {
+		RK_ASSERT (markiface);
+		return;
+	}
+	bool old_rw = m_doc->isReadWrite ();
+	if (!old_rw) m_doc->setReadWrite (true);
+	markiface->addMark (linenum, KTextEditor::MarkInterface::Execution);
+	m_view->setCursorPosition (KTextEditor::Cursor (linenum, 0));
+	if (!old_rw) m_doc->setReadWrite (false);
 }
 
 void RKCommandEditorWindow::updateCaption (KTextEditor::Document*) {
@@ -790,6 +869,7 @@ void RKCommandEditorWindow::selectionChanged (KTextEditor::View* view) {
 //////////////////////// RKFunctionArgHinter //////////////////////////////
 
 #include <QToolTip>
+#include <QStyle>
 
 #include "../core/rfunctionobject.h"
 
@@ -810,10 +890,13 @@ RKFunctionArgHinter::RKFunctionArgHinter (RKScriptContextProvider *provider, KTe
 	QPalette p = QToolTip::palette ();		// HACK to trick the style into using the correct color
 	p.setColor (QPalette::Inactive, QPalette::Window, p.color (QPalette::Inactive, QPalette::ToolTipBase));
 	p.setColor (QPalette::Inactive, QPalette::WindowText, p.color (QPalette::Inactive, QPalette::ToolTipText));
+	arghints_popup->setForegroundRole (QPalette::ToolTipText);
+	arghints_popup->setBackgroundRole (QPalette::ToolTipBase);
 	arghints_popup->setPalette (p);
 	arghints_popup->setFrameStyle (QFrame::Box);
 	arghints_popup->setLineWidth (1);
 	arghints_popup->setWordWrap (true);
+	arghints_popup->setWindowOpacity (arghints_popup->style ()->styleHint (QStyle::SH_ToolTipLabel_Opacity, 0, arghints_popup) / 255.0);
 	arghints_popup->hide ();
 	active = false;
 
@@ -951,25 +1034,23 @@ void RKCodeCompletionModel::updateCompletionList (const QString& symbol) {
 	RObjectList::getObjectList ()->findObjectsMatching (symbol, &map);
 
 	int count = map.size ();
-	list.clear ();
-	list_names.clear ();
-	list.reserve (count);
-	list_names.reserve (count);
-	// this is silly, but we need an int indexable storage, so we copy the map to a list
+	icons.clear ();
+	names.clear ();
+#if QT_VERSION >= 0x040700
+	icons.reserve (count);
+	names.reserve (count);
+#endif
+	// copy the map to two lists. For one thing, we need an int indexable storage, for another, caching this information is safer
+	// in case objects are removed while the completion mode is active.
 	for (RObject::RObjectSearchMap::const_iterator it = map.constBegin (); it != map.constEnd (); ++it) {
-		list.append (it.value ());
-		// the second list is used to store the name that should be used for completion.
-		// This may be object->getBaseName() or object->getFullName () depending on whether the object is
-		// masked or not.
-		list_names.append (it.key ());
+		icons.append (RKStandardIcons::iconForObject (it.value ()));
+		names.append (it.value ()->getBaseName ());
 	}
 
 	setRowCount (count);
 	current_symbol = symbol;
 
 	reset ();
-
-#warning deal with object removals while the completion model is active! Maybe cache all information?
 }
 
 void RKCodeCompletionModel::completionInvoked (KTextEditor::View*, const KTextEditor::Range&, InvocationType) {
@@ -985,9 +1066,9 @@ void RKCodeCompletionModel::completionInvoked (KTextEditor::View*, const KTextEd
 void RKCodeCompletionModel::executeCompletionItem (KTextEditor::Document *document, const KTextEditor::Range &word, int row) const {
 	RK_TRACE (COMMANDEDITOR);
 
-	RK_ASSERT (list_names.size () > row);
+	RK_ASSERT (names.size () > row);
 
-	document->replaceText (word, list_names[row]);
+	document->replaceText (word, names[row]);
 }
 
 QVariant RKCodeCompletionModel::data (const QModelIndex& index, int role) const {
@@ -996,18 +1077,14 @@ QVariant RKCodeCompletionModel::data (const QModelIndex& index, int role) const
 	int row = index.row ();
 
 	if (index.parent ().isValid ()) return QVariant ();
-	if (row >= list.count ()) return QVariant ();
-
-	RObject* object = list[row];
-	RK_ASSERT (object);
 
 	if ((role == Qt::DisplayRole) || (role==KTextEditor::CodeCompletionModel::CompletionRole)) {
 		if (col == KTextEditor::CodeCompletionModel::Name) {
-			return (object->getBaseName ());
+			return (names.value (row));
 		}
 	} else if (role == Qt::DecorationRole) {
 		if (col == KTextEditor::CodeCompletionModel::Icon) {
-			return RKStandardIcons::iconForObject (object);
+			return (icons.value (row));
 		}
 	}
 
@@ -1173,7 +1250,7 @@ void RKCommandHighlighter::setHighlighting (KTextEditor::Document *doc, Highligh
 
 	QString mode_string = "R Script";
 	if (mode == RInteractiveSession) mode_string = "R interactive session";
-	if (!doc->setHighlightingMode (mode_string)) RK_DO (qDebug ("R syntax highlighting defintion ('%s')not found!", qPrintable (mode_string)), COMMANDEDITOR, DL_ERROR);
+	if (!(doc->setHighlightingMode (mode_string) && doc->setMode (mode_string))) RK_DO (qDebug ("R syntax highlighting defintion ('%s')not found!", qPrintable (mode_string)), COMMANDEDITOR, DL_ERROR);
 }
 
 void RKCommandHighlighter::copyLinesToOutput (KTextEditor::View *view, HighlightingMode mode) {
