@@ -136,10 +136,10 @@ public:
 	void executeCompletionItem (KTextEditor::Document *document, const KTextEditor::Range &word, int row) const;
 	QVariant data (const QModelIndex& index, int role=Qt::DisplayRole) const;
 
-	bool isEmpty () const { return list.isEmpty (); };
+	bool isEmpty () const { return names.isEmpty (); };
 private:
-	QVector<RObject*> list;
-	QVector<QString> list_names;
+	QList<QIcon> icons;
+	QStringList names;
 	QString current_symbol;
 	RKCommandEditorWindow *command_editor;
 };
@@ -168,13 +168,17 @@ public:
 @param encoding encoding to use. If QString (), the default encoding is used.
 @param read_only Open the file in read-only mode
 @param delete_on_close File should be deleted when closing the window. Only respected with read_only=true. */
-	bool openURL (const KUrl &url, const QString& encoding=QString (), bool use_r_highlighting=true, bool read_only=false, bool delete_on_close=false);
+	bool openURL (const KUrl url, const QString& encoding=QString (), bool use_r_highlighting=true, bool read_only=false, bool delete_on_close=false);
 /** returns, whether the document was modified since the last save */
 	bool isModified ();
 /** insert the given text into the document at the current cursor position. Additionally, focuses the view */
 	void insertText (const QString &text);
 /** set the current text (clear all previous text, and sets new text) */
 	void setText (const QString &text);
+/** @see restoreScrollPosition (). Note: Currently this saves/restored the cursor position, not necessarily the scroll position. */
+	void saveScrollPosition ();
+/** @see saveScrollPosition (). Note: Currently this saves/restored the cursor position, not necessarily the scroll position. */
+	void restoreScrollPosition ();
 /** copy current selection. Wrapper for use by external classes */
 	void copy ();
 
@@ -188,6 +192,8 @@ public:
 
 	QString provideContext (int line_rev);
 	QString currentCompletionWord () const;
+
+	void highlightLine (int linenum);
 public slots:
 /** update Tab caption according to the current url. Display the filename-component of the URL, or - if not available - a more elaborate description of the url. Also appends a "[modified]" if appropriate */
 	void updateCaption (KTextEditor::Document* = 0);
@@ -241,6 +247,7 @@ private slots:
 /** handle any errors during auto-saving */
 	void autoSaveHandlerJobFinished (RKJobSequence* seq);
 private:
+	KTextEditor::Cursor saved_scroll_position;
 	KTextEditor::Document *m_doc;
 	KTextEditor::View *m_view;
 	KTextEditor::CodeCompletionInterface *cc_iface;
