@@ -2,7 +2,7 @@
                           rkhtmlwindow  -  description
                              -------------------
     begin                : Wed Oct 12 2005
-    copyright            : (C) 2005, 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -22,10 +22,11 @@
 #include <klocale.h>
 #include <kiconloader.h>
 #include <kmessagebox.h>
-#include <kparts/partmanager.h>
+#include <kparts/plugin.h>
 #include <kactioncollection.h>
 #include <kdirwatch.h>
 #include <kmimetype.h>
+#include <kio/job.h>
 
 #include <qfileinfo.h>
 #include <qwidget.h>
@@ -62,10 +63,16 @@ RKHTMLWindow::RKHTMLWindow (QWidget *parent, WindowMode mode) : RKMDIWindow (par
 	khtmlpart = new KHTMLPart (this, 0, KHTMLPart::BrowserViewGUI);
 	setPart (khtmlpart);
 	fixupPartGUI ();
+// WORKAROUND for annoying kdelibs bug in KDE 4.6: https://sourceforge.net/tracker/?func=detail&atid=459007&aid=3310106&group_id=50231
+// NOTE: Fixed in KDE 4.7. See http://git.reviewboard.kde.org/r/101491/
+	QAction *action = khtmlpart->action ("findAheadText");
+	if (action) action->setShortcutContext (Qt::WidgetWithChildrenShortcut);
+// WORKAROUND end
 	initializeActivationSignals ();
 	RKXMLGUISyncer::self()->registerChangeListener (khtmlpart, this, SLOT (fixupPartGUI()));
 	khtmlpart->setSelectable (true);
 	setFocusProxy (khtmlpart->widget ());
+	khtmlpart->widget ()->setFocusPolicy (Qt::StrongFocus);
 	
 	khtmlpart->widget ()->setSizePolicy (QSizePolicy::Expanding, QSizePolicy::Expanding);
 	QHBoxLayout *pLayout = new QHBoxLayout (this);
@@ -75,6 +82,7 @@ RKHTMLWindow::RKHTMLWindow (QWidget *parent, WindowMode mode) : RKMDIWindow (par
 	// We have to connect this in order to allow browsing.
 	connect (khtmlpart->browserExtension (), SIGNAL (openUrlRequestDelayed (const KUrl&, const KParts::OpenUrlArguments&, const KParts::BrowserArguments&)), this, SLOT (slotOpenUrl (const KUrl&, const KParts::OpenUrlArguments&, const KParts::BrowserArguments&)));
 	connect (khtmlpart, SIGNAL (completed ()), this, SLOT (loadDone ()));
+	connect (khtmlpart->browserExtension (), SIGNAL (openUrlNotify()), this, SLOT (internalNavigation()));	// to catch internal navigation on a page
 
 	current_history_position = -1;
 	url_change_is_from_history = false;
@@ -87,6 +95,16 @@ RKHTMLWindow::RKHTMLWindow (QWidget *parent, WindowMode mode) : RKMDIWindow (par
 RKHTMLWindow::~RKHTMLWindow () {
 	RK_TRACE (APP);
 
+// WORKAROUND for annoying kdelibs bug (KDE 4.0 up to at least KDE 4.6): Status bar icons added by plugins typically do not get deleted in case the KParts::StatusBarExtension
+// has already been deleted, first. See http://www.mail-archive.com/rkward-devel@lists.sourceforge.net/msg01345.html . Therefore, delete the plugins, explicitely, while the
+// StatusBarExtension is still alive...
+	QList<KParts::Plugin*> plugins = KParts::Plugin::pluginObjects (khtmlpart);
+	foreach (KParts::Plugin *plugin, plugins) {
+		delete plugin;
+	}
+// I hope this does not come back to bite us one day... If it does, here's a safer variant, which simply hides the problem (the way it is hidden in konqueror, among others):
+// 	RKWardMainWindow::getMain ()->partManager ()->setActivePart (0);
+// WORKAROUND end
 	delete khtmlpart;
 }
 
@@ -175,35 +193,45 @@ void RKHTMLWindow::slotPrint () {
 	khtmlpart->view ()->print ();
 }
 
-void RKHTMLWindow::slotForward () {
+void RKHTMLWindow::openLocationFromHistory (const VisitedLocation &loc) {
 	RK_TRACE (APP);
 	RK_ASSERT (window_mode == HTMLHelpWindow);
 
-	url_change_is_from_history = true;
+	int history_last = url_history.count () - 1;
+	RK_ASSERT (current_history_position >= 0);
+	RK_ASSERT (current_history_position <= history_last);
+	if (loc.url == khtmlpart->url ()) {
+		khtmlpart->view()->setContentsPos (0, loc.y_offset);
+	} else {
+		url_change_is_from_history = true;
+		KParts::OpenUrlArguments args;
+		args.setYOffset (loc.y_offset);
+		khtmlpart->setArguments (args);
+		openURL (loc.url);
+		url_change_is_from_history = false;
+	}
+
+	back->setEnabled (current_history_position > 0);
+	forward->setEnabled (current_history_position < history_last);
+}
+
+void RKHTMLWindow::slotForward () {
+	RK_TRACE (APP);
 
 	++current_history_position;
-	int history_last = url_history.count () - 1;
-	RK_ASSERT (current_history_position > 0);
-	RK_ASSERT (current_history_position <= history_last);
-	openURL (url_history[current_history_position]);
-
-	back->setEnabled (true);
-	forward->setEnabled (current_history_position < history_last);
-	url_change_is_from_history = false;
+	openLocationFromHistory (url_history[current_history_position]);
 }
 
 void RKHTMLWindow::slotBack () {
 	RK_TRACE (APP);
-	RK_ASSERT (window_mode == HTMLHelpWindow);
 
-	url_change_is_from_history = true;
+	// if going back from the end of the history, save that position, first.
+	if (current_history_position >= (url_history.count () - 1)) {
+		changeURL (khtmlpart->url ());
+		--current_history_position;
+	}
 	--current_history_position;
-	RK_ASSERT (current_history_position >= 0);
-	openURL (url_history[current_history_position]);
-
-	forward->setEnabled (true);
-	back->setEnabled (current_history_position > 0);
-	url_change_is_from_history = false;
+	openLocationFromHistory (url_history[current_history_position]);
 }
 
 bool RKHTMLWindow::handleRKWardURL (const KUrl &url) {
@@ -217,13 +245,16 @@ bool RKHTMLWindow::handleRKWardURL (const KUrl &url) {
 			RKComponentMap::invokeComponent (path.left (sep), path.mid (sep+1).split ('\n', QString::SkipEmptyParts));
 			return true;
 		} else {
-			bool ok = false;
-			if (url.host () == "component") {
-				ok = renderRKHelp (url);
-			} else if (url.host () == "rhelp") {
+			if (url.host () == "rhelp") {
 				// TODO: find a nice solution to render this in the current window
 				RKHelpSearchWindow::mainHelpSearch ()->getFunctionHelp (url.path ().mid (1));
 				return true;
+			}
+
+			changeURL (url);
+			bool ok = false;
+			if (url.host () == "component") {
+				ok = renderRKHelp (url);
 			} else if (url.host () == "page") {
 				ok = renderRKHelp (url);
 			} else if (url.host ().toUpper () == "RHELPBASE") {	// NOTE: QUrl () may lowercase the host part, internally
@@ -233,12 +264,10 @@ bool RKHTMLWindow::handleRKWardURL (const KUrl &url) {
 				if (url.hasFragment ()) fixed_url.setFragment (url.fragment ());
 				ok = openURL (fixed_url);
 			}
-		
+
 			if (!ok) {
 				fileDoesNotExistMessage ();
 			}
-		
-			changeURL (url);
 			return true;
 		}
 	}
@@ -249,6 +278,7 @@ bool RKHTMLWindow::openURL (const KUrl &url) {
 	RK_TRACE (APP);
 
 	if (handleRKWardURL (url)) return true;
+
 	if (window_mode == HTMLOutputWindow) {
 		if (url != current_url) {
 			// output window should not change url after initialization
@@ -260,55 +290,82 @@ bool RKHTMLWindow::openURL (const KUrl &url) {
 			current_url = url;	// needs to be set before registering
 			RKOutputWindowManager::self ()->registerWindow (this);
 		}
-	} else {
-		if (!(url.isLocalFile ())) {
-			if (window_mode == HTMLHelpWindow) {
-				// since R 2.10.0, help urls may be on local ports
-				if (url.protocol ().toLower ().startsWith ("http")) {
-					QString host = url.host ();
-					if ((host == "127.0.0.1") || (host == "localhost") || host == QHostInfo::localHostName ()) {
-						khtmlpart->openUrl (url);
-						changeURL (url);
-						return true;
-					}
-				}
-			}
+	}
+
+	if (url.isLocalFile () && (KMimeType::findByUrl (url)->is ("text/html") || window_mode == HTMLOutputWindow)) {
+		changeURL (url);
+		QFileInfo out_file (url.toLocalFile ());
+		bool ok = out_file.exists();
+		if (ok)  {
+			khtmlpart->openUrl (url);
+		} else {
+			fileDoesNotExistMessage ();
 		}
-		if (!(url.isLocalFile () && KMimeType::findByUrl (url)->is ("text/html"))) {
-			RKWorkplace::mainWorkplace ()->openAnyUrl (url);
+		return ok;
+	}
+
+	if (url_change_is_from_history || url.protocol ().toLower ().startsWith ("help")) {	// handle help pages, and any page that we have previously handled (from history)
+		changeURL (url);
+		khtmlpart->openUrl (url);
+		return true;
+	}
+
+	// special casing for R's dynamic help pages. These should be considered local, even though they are served through http
+	if (url.protocol ().toLower ().startsWith ("http")) {
+		QString host = url.host ();
+		if ((host == "127.0.0.1") || (host == "localhost") || host == QHostInfo::localHostName ()) {
+			KIO::TransferJob *job = KIO::get (url, KIO::Reload);
+			connect (job, SIGNAL (mimetype(KIO::Job*, const QString&)), this, SLOT (mimeTypeDetermined(KIO::Job*, const QString&)));
 			return true;
 		}
 	}
 
-	QFileInfo out_file (url.toLocalFile ());
-	bool ok = out_file.exists();
-	if (ok)  {
-		khtmlpart->openUrl (url);
-	} else {
-		fileDoesNotExistMessage ();
-	}
-	changeURL (url);
-	return ok;
+	RKWorkplace::mainWorkplace ()->openAnyUrl (url, QString (), KMimeType::findByUrl (url)->is ("text/html"));	// NOTE: text/html type urls, which we have not handled, above, are forced to be opened externally, to avoid recursion. E.g. help:// protocol urls.
+	return true;
 }
 
 KUrl RKHTMLWindow::url () {
 	return current_url;
 }
 
+void RKHTMLWindow::mimeTypeDetermined (KIO::Job* job, const QString& type) {
+	RK_TRACE (APP);
+
+	KIO::TransferJob* tj = static_cast<KIO::TransferJob*> (job);
+	KUrl url = tj->url ();
+	tj->putOnHold ();
+	if (type == "text/html") {
+		changeURL (url);
+		khtmlpart->openUrl (url);
+	} else {
+		RKWorkplace::mainWorkplace ()->openAnyUrl (url, type);
+	}
+}
+
+void RKHTMLWindow::internalNavigation () {
+	RK_TRACE (APP);
+
+	changeURL (khtmlpart->url ());
+}
+
 void RKHTMLWindow::changeURL (const KUrl &url) {
 	current_url = url;
 	updateCaption (url);
 
 	if (!url_change_is_from_history) {
 		if (window_mode == HTMLHelpWindow) {
-			int history_last = url_history.count () - 1;
-			for (int i = history_last; i > current_history_position; --i) {
-				url_history.removeLast ();
+			if (current_history_position >= 0) {	// skip initial blank page
+				url_history = url_history.mid (0, current_history_position);
+
+				VisitedLocation loc;
+				loc.url = khtmlpart->url ();
+				KParts::OpenUrlArguments args = khtmlpart->arguments ();
+				loc.y_offset = args.yOffset ();
+				url_history.append (loc);
 			}
 
-			url_history.append (url);
 			++current_history_position;
-			back->setEnabled (current_history_position > 0);	// may be false, if this is the very first page to be added to the history
+ 			back->setEnabled (current_history_position > 0);
 			forward->setEnabled (false);
 		}
 	}
@@ -345,6 +402,7 @@ void RKHTMLWindow::loadDone () {
 		khtmlpart->view ()->setContentsPos (0, khtmlpart->view ()->contentsHeight ());
 	} else {	// scroll to previous pos
 		if (scroll_position >= 0) khtmlpart->view()->setContentsPos (0, scroll_position);
+		scroll_position = -1;
 	}
 }
 
