@@ -2,7 +2,7 @@
                           rwindowcatcher.cpp  -  description
                              -------------------
     begin                : Wed May 4 2005
-    copyright            : (C) 2005, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2006, 2007, 2009, 2010, 2011, 2012 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -66,7 +66,11 @@ void RKWindowCatcher::stop (int new_cur_device) {
 			KWindowInfo wininfo = KWindowSystem::windowInfo (w, NET::WMName | NET::WMGeometry);
 #endif
 		} else {
+#if defined Q_WS_MAC
+			KMessageBox::information (0, i18n ("You have created a new graphics device window in R. Usually, RKWard tries to detect such windows, to take control of them, and add a menu-bar to them. However, this is not currently supported in this build of RKWard on Mac OS X. See http://p.sf.net/rkward/mac for more information."), i18n ("Could not embed R X11 window"), "embed_x11_device_not_supported");
+#else
 			KMessageBox::information (0, i18n ("You have created a new X11 device window in R. Usually, RKWard tries to detect such windows, to take control of them, and add a menu-bar to them. This time, however, RKWard failed to detect, which window was created, and so can not embed it.\nIf you created the window on a different screen or X11 display, that is to be expected. You might want to consider changing options(\"display\"), then.\nIf you can see the X11 window on the same screen as this message, then RKWard should do better. In this case, please contact us at rkward-devel@lists.sourceforge.net with details on your setup, so we can try to fix this in future versions of RKWard."), i18n ("Could not embed R X11 window"), "failure_to_detect_x11_device");
+#endif
 		}
 	}
 	last_cur_device = new_cur_device;
@@ -223,7 +227,7 @@ void RKCaughtX11Window::doEmbed () {
 	QRect dims = window ()->frameGeometry ();
 	QRect avail = QApplication::desktop ()->availableGeometry (window ());
 	if ((dims.width () > avail.width ()) || (dims.height () > avail.height ())) {
-		KMessageBox::information (this, i18n ("The current window appears to too large to fit on the screen. If this happens regularly, you may want to adjust the default graphics window size in Settings->Configure RKWard->Onscreen Graphics."), i18n ("Large window"), "dont_ask_again_large_x11_window");
+		KMessageBox::information (this, i18n ("The current window appears too large to fit on the screen. If this happens regularly, you may want to adjust the default graphics window size in Settings->Configure RKWard->Onscreen Graphics."), i18n ("Large window"), "dont_ask_again_large_x11_window");
 	}
 }
 
@@ -517,6 +521,15 @@ void RKCaughtX11Window::setStatusMessage (const QString& message, RCommand *comm
 	}
 }
 
+// static
+void RKCaughtX11Window::setStatusMessage(int dev_num, const QString& message, RCommand* command) {
+	RK_TRACE (MISC);
+
+	RKCaughtX11Window *window = getWindow (dev_num);
+	if (!window) return;
+	window->setStatusMessage (message, command);
+}
+
 void RKCaughtX11Window::rCommandDone (RCommand *command) {
 	RK_TRACE (MISC);
 
